@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,9 +25,10 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering;
 
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Random;
 
 import de.lmu.ifi.dbs.elki.algorithm.AbstractAlgorithm;
+import de.lmu.ifi.dbs.elki.algorithm.clustering.kmeans.KMeansInitialization;
+import de.lmu.ifi.dbs.elki.algorithm.clustering.kmeans.RandomlyGeneratedInitialMeans;
 import de.lmu.ifi.dbs.elki.data.Cluster;
 import de.lmu.ifi.dbs.elki.data.Clustering;
 import de.lmu.ifi.dbs.elki.data.NumberVector;
@@ -38,10 +39,12 @@ import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.MathUtil;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
@@ -58,8 +61,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualCons
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.LongParameter;
-import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
 
 /**
  * Provides the EM algorithm (clustering by expectation maximization).
@@ -113,6 +115,11 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
    */
   public static final OptionID DELTA_ID = OptionID.getOrCreateOptionID("em.delta", "The termination criterion for maximization of E(M): " + "E(M) - E(M') < em.delta");
 
+  /**
+   * Parameter to specify the initialization method
+   */
+  public static final OptionID INIT_ID = OptionID.getOrCreateOptionID("kmeans.initialization", "Method to choose the initial means.");
+
   private static final double MIN_LOGLIKELIHOOD = -100000;
 
   /**
@@ -120,33 +127,28 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
    */
   private double delta;
 
-  /**
-   * Parameter to specify the random generator seed.
-   */
-  public static final OptionID SEED_ID = OptionID.getOrCreateOptionID("em.seed", "The random number generator seed.");
-
-  /**
-   * Holds the value of {@link #SEED_ID}.
-   */
-  private Long seed;
-
   /**
    * Store the individual probabilities, for use by EMOutlierDetection etc.
    */
   private WritableDataStore<double[]> probClusterIGivenX;
 
+  /**
+   * Class to choose the initial means
+   */
+  private KMeansInitialization<V> initializer;
+
   /**
    * Constructor.
    * 
    * @param k k parameter
    * @param delta delta parameter
-   * @param seed Seed parameter
+   * @param initializer Class to choose the initial means
    */
-  public EM(int k, double delta, Long seed) {
+  public EM(int k, double delta, KMeansInitialization<V> initializer) {
     super();
     this.k = k;
     this.delta = delta;
-    this.seed = seed;
+    this.initializer = initializer;
   }
 
   /**
@@ -169,28 +171,31 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
     if(logger.isVerbose()) {
       logger.verbose("initializing " + k + " models");
     }
-    List<V> means = initialMeans(relation);
+    List<Vector> means = new ArrayList<Vector>();
+    for(NumberVector<?, ?> nv : initializer.chooseInitialMeans(relation, k, EuclideanDistanceFunction.STATIC)) {
+      means.add(nv.getColumnVector());
+    }
     List<Matrix> covarianceMatrices = new ArrayList<Matrix>(k);
-    List<Double> normDistrFactor = new ArrayList<Double>(k);
+    double[] normDistrFactor = new double[k];
     List<Matrix> invCovMatr = new ArrayList<Matrix>(k);
-    List<Double> clusterWeights = new ArrayList<Double>(k);
+    double[] clusterWeights = new double[k];
     probClusterIGivenX = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_SORTED, double[].class);
 
-    int dimensionality = means.get(0).getDimensionality();
+    final int dimensionality = means.get(0).getDimensionality();
     for(int i = 0; i < k; i++) {
       Matrix m = Matrix.identity(dimensionality, dimensionality);
       covarianceMatrices.add(m);
-      normDistrFactor.add(1.0 / Math.sqrt(Math.pow(MathUtil.TWOPI, dimensionality) * m.det()));
+      normDistrFactor[i] = 1.0 / Math.sqrt(Math.pow(MathUtil.TWOPI, dimensionality) * m.det());
       invCovMatr.add(m.inverse());
-      clusterWeights.add(1.0 / k);
+      clusterWeights[i] = 1.0 / k;
       if(logger.isDebuggingFinest()) {
         StringBuffer msg = new StringBuffer();
         msg.append(" model ").append(i).append(":\n");
         msg.append(" mean:    ").append(means.get(i)).append("\n");
         msg.append(" m:\n").append(FormatUtil.format(m, "        ")).append("\n");
         msg.append(" m.det(): ").append(m.det()).append("\n");
-        msg.append(" cluster weight: ").append(clusterWeights.get(i)).append("\n");
-        msg.append(" normDistFact:   ").append(normDistrFactor.get(i)).append("\n");
+        msg.append(" cluster weight: ").append(clusterWeights[i]).append("\n");
+        msg.append(" normDistFact:   ").append(normDistrFactor[i]).append("\n");
         logger.debugFine(msg.toString());
       }
     }
@@ -211,46 +216,45 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
       em = emNew;
 
       // recompute models
-      List<V> meanSums = new ArrayList<V>(k);
+      List<Vector> meanSums = new ArrayList<Vector>(k);
       double[] sumOfClusterProbabilities = new double[k];
 
       for(int i = 0; i < k; i++) {
-        clusterWeights.set(i, 0.0);
-        meanSums.add(means.get(i).nullVector());
+        clusterWeights[i] = 0.0;
+        meanSums.add(new Vector(dimensionality));
         covarianceMatrices.set(i, Matrix.zeroMatrix(dimensionality));
       }
 
       // weights and means
-      for(DBID id : relation.iterDBIDs()) {
-        double[] clusterProbabilities = probClusterIGivenX.get(id);
+      for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+        double[] clusterProbabilities = probClusterIGivenX.get(iditer);
 
         for(int i = 0; i < k; i++) {
           sumOfClusterProbabilities[i] += clusterProbabilities[i];
-          V summand = relation.get(id).multiplicate(clusterProbabilities[i]);
-          V currentMeanSum = meanSums.get(i).plus(summand);
-          meanSums.set(i, currentMeanSum);
+          Vector summand = relation.get(iditer).getColumnVector().timesEquals(clusterProbabilities[i]);
+          meanSums.get(i).plusEquals(summand);
         }
       }
       final int n = relation.size();
       for(int i = 0; i < k; i++) {
-        clusterWeights.set(i, sumOfClusterProbabilities[i] / n);
-        V newMean = meanSums.get(i).multiplicate(1 / sumOfClusterProbabilities[i]);
+        clusterWeights[i] = sumOfClusterProbabilities[i] / n;
+        Vector newMean = meanSums.get(i).timesEquals(1 / sumOfClusterProbabilities[i]);
         means.set(i, newMean);
       }
       // covariance matrices
-      for(DBID id : relation.iterDBIDs()) {
-        double[] clusterProbabilities = probClusterIGivenX.get(id);
-        V instance = relation.get(id);
+      for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+        double[] clusterProbabilities = probClusterIGivenX.get(iditer);
+        Vector instance = relation.get(iditer).getColumnVector();
         for(int i = 0; i < k; i++) {
-          V difference = instance.minus(means.get(i));
-          covarianceMatrices.get(i).plusEquals(difference.getColumnVector().times(difference.getRowVector()).times(clusterProbabilities[i]));
+          Vector difference = instance.minus(means.get(i));
+          covarianceMatrices.get(i).plusEquals(difference.timesTranspose(difference).timesEquals(clusterProbabilities[i]));
         }
       }
       for(int i = 0; i < k; i++) {
         covarianceMatrices.set(i, covarianceMatrices.get(i).times(1 / sumOfClusterProbabilities[i]).cheatToAvoidSingularity(SINGULARITY_CHEAT));
       }
       for(int i = 0; i < k; i++) {
-        normDistrFactor.set(i, 1.0 / Math.sqrt(Math.pow(MathUtil.TWOPI, dimensionality) * covarianceMatrices.get(i).det()));
+        normDistrFactor[i] = 1.0 / Math.sqrt(Math.pow(MathUtil.TWOPI, dimensionality) * covarianceMatrices.get(i).det());
         invCovMatr.set(i, covarianceMatrices.get(i).inverse());
       }
       // reassign probabilities
@@ -269,8 +273,8 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
     }
 
     // provide a hard clustering
-    for(DBID id : relation.iterDBIDs()) {
-      double[] clusterProbabilities = probClusterIGivenX.get(id);
+    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      double[] clusterProbabilities = probClusterIGivenX.get(iditer);
       int maxIndex = 0;
       double currentMax = 0.0;
       for(int i = 0; i < k; i++) {
@@ -279,15 +283,16 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
           currentMax = clusterProbabilities[i];
         }
       }
-      hardClusters.get(maxIndex).add(id);
+      hardClusters.get(maxIndex).add(iditer);
     }
+    final V factory = DatabaseUtil.assumeVectorField(relation).getFactory();
     Clustering<EMModel<V>> result = new Clustering<EMModel<V>>("EM Clustering", "em-clustering");
     // provide models within the result
     for(int i = 0; i < k; i++) {
       // TODO: re-do labeling.
       // SimpleClassLabel label = new SimpleClassLabel();
       // label.init(result.canonicalClusterLabel(i));
-      Cluster<EMModel<V>> model = new Cluster<EMModel<V>>(hardClusters.get(i), new EMModel<V>(means.get(i), covarianceMatrices.get(i)));
+      Cluster<EMModel<V>> model = new Cluster<EMModel<V>>(hardClusters.get(i), new EMModel<V>(factory.newNumberVector(means.get(i).getArrayRef()), covarianceMatrices.get(i)));
       result.addCluster(model);
     }
     return result;
@@ -308,29 +313,25 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
    * @param clusterWeights the weights of the current clusters
    * @return the expectation value of the current mixture of distributions
    */
-  protected double assignProbabilitiesToInstances(Relation<V> database, List<Double> normDistrFactor, List<V> means, List<Matrix> invCovMatr, List<Double> clusterWeights, WritableDataStore<double[]> probClusterIGivenX) {
+  protected double assignProbabilitiesToInstances(Relation<V> database, double[] normDistrFactor, List<Vector> means, List<Matrix> invCovMatr, double[] clusterWeights, WritableDataStore<double[]> probClusterIGivenX) {
     double emSum = 0.0;
 
-    for(DBID id : database.iterDBIDs()) {
-      V x = database.get(id);
-      List<Double> probabilities = new ArrayList<Double>(k);
+    for(DBIDIter iditer = database.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      Vector x = database.get(iditer).getColumnVector();
+      double[] probabilities = new double[k];
       for(int i = 0; i < k; i++) {
-        V difference = x.minus(means.get(i));
-        Matrix differenceRow = difference.getRowVector();
-        Vector differenceCol = difference.getColumnVector();
-        Matrix rowTimesCov = differenceRow.times(invCovMatr.get(i));
-        Vector rowTimesCovTimesCol = rowTimesCov.times(differenceCol);
-        double power = rowTimesCovTimesCol.get(0, 0) / 2.0;
-        double prob = normDistrFactor.get(i) * Math.exp(-power);
+        Vector difference = x.minus(means.get(i));
+        double rowTimesCovTimesCol = difference.transposeTimesTimes(invCovMatr.get(i), difference);
+        double power = rowTimesCovTimesCol / 2.0;
+        double prob = normDistrFactor[i] * Math.exp(-power);
         if(logger.isDebuggingFinest()) {
-          logger.debugFinest(" difference vector= ( " + difference.toString() + " )\n" + " differenceRow:\n" + FormatUtil.format(differenceRow, "    ") + "\n" + " differenceCol:\n" + FormatUtil.format(differenceCol, "    ") + "\n" + " rowTimesCov:\n" + FormatUtil.format(rowTimesCov, "    ") + "\n" + " rowTimesCovTimesCol:\n" + FormatUtil.format(rowTimesCovTimesCol, "    ") + "\n" + " power= " + power + "\n" + " prob=" + prob + "\n" + " inv cov matrix: \n" + FormatUtil.format(invCovMatr.get(i), "     "));
+          logger.debugFinest(" difference vector= ( " + difference.toString() + " )\n" + " difference:\n" + FormatUtil.format(difference, "    ") + "\n" + " rowTimesCovTimesCol:\n" + rowTimesCovTimesCol + "\n" + " power= " + power + "\n" + " prob=" + prob + "\n" + " inv cov matrix: \n" + FormatUtil.format(invCovMatr.get(i), "     "));
         }
-
-        probabilities.add(prob);
+        probabilities[i] = prob;
       }
       double priorProbability = 0.0;
       for(int i = 0; i < k; i++) {
-        priorProbability += probabilities.get(i) * clusterWeights.get(i);
+        priorProbability += probabilities[i] * clusterWeights[i];
       }
       double logP = Math.max(Math.log(priorProbability), MIN_LOGLIKELIHOOD);
       if(!Double.isNaN(logP)) {
@@ -340,70 +341,28 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
       double[] clusterProbabilities = new double[k];
       for(int i = 0; i < k; i++) {
         assert (priorProbability >= 0.0);
-        assert (clusterWeights.get(i) >= 0.0);
+        assert (clusterWeights[i] >= 0.0);
         // do not divide by zero!
         if(priorProbability == 0.0) {
           clusterProbabilities[i] = 0.0;
         }
         else {
-          clusterProbabilities[i] = probabilities.get(i) / priorProbability * clusterWeights.get(i);
+          clusterProbabilities[i] = probabilities[i] / priorProbability * clusterWeights[i];
         }
       }
-      probClusterIGivenX.put(id, clusterProbabilities);
+      probClusterIGivenX.put(iditer, clusterProbabilities);
     }
 
     return emSum;
   }
 
-  /**
-   * Creates {@link #k k} random points distributed uniformly within the
-   * attribute ranges of the given database.
-   * 
-   * @param relation the database must contain enough points in order to
-   *        ascertain the range of attribute values. Less than two points would
-   *        make no sense. The content of the database is not touched otherwise.
-   * @return a list of {@link #k k} random points distributed uniformly within
-   *         the attribute ranges of the given database
-   */
-  protected List<V> initialMeans(Relation<V> relation) {
-    final Random random;
-    if(this.seed != null) {
-      random = new Random(this.seed);
-    }
-    else {
-      random = new Random();
-    }
-    if(relation.size() > 0) {
-      final int dim = DatabaseUtil.dimensionality(relation);
-      Pair<V, V> minmax = DatabaseUtil.computeMinMax(relation);
-      List<V> means = new ArrayList<V>(k);
-      if(logger.isVerbose()) {
-        logger.verbose("initializing random vectors");
-      }
-      for(int i = 0; i < k; i++) {
-        double[] r = MathUtil.randomDoubleArray(dim, random);
-        // Rescale
-        for (int d = 0; d < dim; d++) {
-          r[d] = minmax.first.doubleValue(d + 1) + (minmax.second.doubleValue(d + 1) - minmax.first.doubleValue(d + 1)) * r[d];
-        }
-        // Instantiate
-        V randomVector = minmax.first.newInstance(r);
-        means.add(randomVector);
-      }
-      return means;
-    }
-    else {
-      return new ArrayList<V>(0);
-    }
-  }
-
   /**
    * Get the probabilities for a given point.
    * 
    * @param index Point ID
    * @return Probabilities of given point
    */
-  public double[] getProbClusterIGivenX(DBID index) {
+  public double[] getProbClusterIGivenX(DBIDRef index) {
     return probClusterIGivenX.get(index);
   }
 
@@ -429,7 +388,7 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
 
     protected double delta;
 
-    protected Long seed;
+    protected KMeansInitialization<V> initializer;
 
     @Override
     protected void makeOptions(Parameterization config) {
@@ -439,20 +398,20 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
         k = kP.getValue();
       }
 
+      ObjectParameter<KMeansInitialization<V>> initialP = new ObjectParameter<KMeansInitialization<V>>(INIT_ID, KMeansInitialization.class, RandomlyGeneratedInitialMeans.class);
+      if(config.grab(initialP)) {
+        initializer = initialP.instantiateClass(config);
+      }
+
       DoubleParameter deltaP = new DoubleParameter(DELTA_ID, new GreaterEqualConstraint(0.0), 0.0);
       if(config.grab(deltaP)) {
         delta = deltaP.getValue();
       }
-
-      LongParameter seedP = new LongParameter(SEED_ID, true);
-      if(config.grab(seedP)) {
-        seed = seedP.getValue();
-      }
     }
 
     @Override
     protected EM<V> makeInstance() {
-      return new EM<V>(k, delta, seed);
+      return new EM<V>(k, delta, initializer);
     }
   }
 }
\ No newline at end of file
