@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -27,7 +27,6 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
@@ -49,6 +48,8 @@ import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableRecordStore;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDMIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
@@ -133,18 +134,19 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
   public Result run(Database database, Relation<O> relation) {
     DistanceQuery<O, D> distQuery = database.getDistanceQuery(relation, getDistanceFunction());
     Class<D> distCls = (Class<D>) getDistanceFunction().getDistanceFactory().getClass();
-    WritableRecordStore store = DataStoreUtil.makeRecordStorage(distQuery.getRelation().getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC, DBID.class, distCls);
+    WritableRecordStore store = DataStoreUtil.makeRecordStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC, DBID.class, distCls);
     pi = store.getStorage(0, DBID.class);
     lambda = store.getStorage(1, distCls);
     // Temporary storage for m.
-    WritableDataStore<D> m = DataStoreUtil.makeStorage(distQuery.getRelation().getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_TEMP, distCls);
+    WritableDataStore<D> m = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_TEMP, distCls);
 
-    FiniteProgress progress = logger.isVerbose() ? new FiniteProgress("Clustering", distQuery.getRelation().size(), logger) : null;
+    FiniteProgress progress = logger.isVerbose() ? new FiniteProgress("Clustering", relation.size(), logger) : null;
     // has to be an array for monotonicity reasons!
-    ModifiableDBIDs processedIDs = DBIDUtil.newArray(distQuery.getRelation().size());
+    ModifiableDBIDs processedIDs = DBIDUtil.newArray(relation.size());
 
     // apply the algorithm
-    for(DBID id : distQuery.getRelation().iterDBIDs()) {
+    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      DBID id  = iditer.getDBID();
       step1(id);
       step2(id, processedIDs, distQuery, m);
       step3(id, processedIDs, m);
@@ -168,8 +170,8 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
     BasicResult result = null;
 
     // Build clusters identified by their target object
-    int minc = minclusters != null ? minclusters : distQuery.getRelation().size();
-    result = extractClusters(distQuery.getRelation().getDBIDs(), pi, lambda, minc);
+    int minc = minclusters != null ? minclusters : relation.size();
+    result = extractClusters(relation.getDBIDs(), pi, lambda, minc);
 
     result.addChildResult(new MaterializedRelation<DBID>("SLINK pi", "slink-order", TypeUtil.DBID, pi, processedIDs));
     result.addChildResult(new MaterializedRelation<D>("SLINK lambda", "slink-order", new SimpleTypeInformation<D>(distCls), lambda, processedIDs));
@@ -200,7 +202,8 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
    * @param distFunc Distance function to use
    */
   private void step2(DBID newID, DBIDs processedIDs, DistanceQuery<O, D> distFunc, WritableDataStore<D> m) {
-    for(DBID id : processedIDs) {
+    for(DBIDIter it = processedIDs.iter(); it.valid(); it.advance()) {
+      DBID id = it.getDBID();
       // M(i) = dist(i, n+1)
       m.put(id, distFunc.distance(id, newID));
     }
@@ -215,7 +218,8 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
    */
   private void step3(DBID newID, DBIDs processedIDs, WritableDataStore<D> m) {
     // for i = 1..n
-    for(DBID id : processedIDs) {
+    for(DBIDIter it = processedIDs.iter(); it.valid(); it.advance()) {
+      DBID id = it.getDBID();
       D l_i = lambda.get(id);
       D m_i = m.get(id);
       DBID p_i = pi.get(id);
@@ -247,7 +251,8 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
    */
   private void step4(DBID newID, DBIDs processedIDs) {
     // for i = 1..n
-    for(DBID id : processedIDs) {
+    for(DBIDIter it = processedIDs.iter(); it.valid(); it.advance()) {
+      DBID id = it.getDBID();
       D l_i = lambda.get(id);
       D lp_i = lambda.get(pi.get(id));
 
@@ -288,7 +293,7 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
     D stopdist = null;
     // sort by lambda
     ArrayModifiableDBIDs order = DBIDUtil.newArray(ids);
-    Collections.sort(order, new CompareByLambda<D>(lambda));
+    order.sort(new CompareByLambda<D>(lambda));
     int index = ids.size() - minclusters - 1;
     while(index >= 0) {
       if(lambda.get(order.get(index)).equals(lambda.get(order.get(index + 1)))) {
@@ -303,7 +308,8 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
     // extract the child clusters
     Map<DBID, ModifiableDBIDs> cluster_ids = new HashMap<DBID, ModifiableDBIDs>();
     Map<DBID, D> cluster_distances = new HashMap<DBID, D>();
-    for(DBID id : ids) {
+    for(DBIDIter it = ids.iter(); it.valid(); it.advance()) {
+      DBID id = it.getDBID();
       DBID lastObjectInCluster = lastObjectInCluster(id, stopdist, pi, lambda);
       ModifiableDBIDs cluster = cluster_ids.get(lastObjectInCluster);
       if(cluster == null) {
@@ -387,7 +393,7 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
       }
       // right child
       DBID rightID = pi.get(leftID);
-      if(leftID.equals(rightID)) {
+      if(leftID.sameDBID(rightID)) {
         break;
       }
       Cluster<DendrogramModel<D>> right = nodes.get(rightID);
@@ -458,7 +464,7 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
     // extract a hierarchical clustering
     ArrayModifiableDBIDs order = DBIDUtil.newArray(ids);
     // sort by lambda
-    Collections.sort(order, new CompareByLambda<D>(lambda));
+    order.sort(new CompareByLambda<D>(lambda));
     D curdist = null;
 
     D stopdist = null;
@@ -472,11 +478,12 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
 
     FiniteProgress progress = logger.isVerbose() ? new FiniteProgress("Extracting clusters", ids.size(), logger) : null;
 
-    for(DBID cur : order) {
-      DBID dest = pi.get(cur);
-      D l = lambda.get(cur);
+    for(DBIDIter it = order.iter(); it.valid(); it.advance()) {
+      DBID dest = pi.get(it);
+      D l = lambda.get(it);
       // logger.debugFine("DBID " + cur.toString() + " dist: " + l.toString());
       if(stopdist != null && stopdist.compareTo(l) > 0) {
+        DBID cur = it.getDBID();
         ModifiableDBIDs curset = cids.remove(cur);
         ModifiableDBIDs destset = cids.get(dest);
         if(destset == null) {
@@ -511,13 +518,11 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
             Cluster<Model> cluster = new Cluster<Model>(cname, clusids, ClusterModel.CLUSTER, hier);
             // Collect child clusters and clean up the cluster ids, keeping only
             // "new" objects.
-            Iterator<DBID> iter = clusids.iterator();
-            while(iter.hasNext()) {
-              DBID child = iter.next();
-              Cluster<Model> chiclus = clusters.get(child);
+            for(DBIDMIter iter = clusids.iter(); iter.valid(); iter.advance()) {
+              Cluster<Model> chiclus = clusters.get(iter);
               if(chiclus != null) {
                 hier.add(cluster, chiclus);
-                clusters.remove(child);
+                clusters.remove(iter);
                 iter.remove();
               }
             }
@@ -545,7 +550,7 @@ public class SLINK<O, D extends Distance<D>> extends AbstractDistanceBasedAlgori
             cids.put(dest, destset);
             destset.add(dest);
           }
-          destset.add(cur);
+          destset.add(it);
         }
       }
       // Decrement counter
