@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.correlation;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -48,14 +48,14 @@ import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.ProxyDatabase;
 import de.lmu.ifi.dbs.elki.database.QueryUtil;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
-import de.lmu.ifi.dbs.elki.datasource.filter.NonNumericFeaturesException;
+import de.lmu.ifi.dbs.elki.datasource.filter.normalization.NonNumericFeaturesException;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.WeightedDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
@@ -84,7 +84,12 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 
 /**
  * Provides the CASH algorithm, an subspace clustering algorithm based on the
- * hough transform.
+ * Hough transform.
+ * 
+ * <b>Note:</b> CASH requires explicitly setting the input vector type to
+ * {@link ParameterizationFunction}:
+ * (in the MiniGui, set option: parser.vector-type ParameterizationFunction).
+ * 
  * <p>
  * Reference: E. Achtert, C. Böhm, J. David, P. Kröger, A. Zimek: Robust
  * clustering in arbitrarily oriented subspaces. <br>
@@ -99,7 +104,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
  */
 // todo elke hierarchy (later)
 @Title("CASH: Robust clustering in arbitrarily oriented subspaces")
-@Description("Subspace clustering algorithm based on the hough transform.")
+@Description("Subspace clustering algorithm based on the Hough transform.")
 @Reference(authors = "E. Achtert, C. Böhm, J. David, P. Kröger, A. Zimek", title = "Robust clustering in arbitraily oriented subspaces", booktitle = "Proc. 8th SIAM Int. Conf. on Data Mining (SDM'08), Atlanta, GA, 2008", url = "http://www.siam.org/proceedings/datamining/2008/dm08_69_AchtertBoehmDavidKroegerZimek.pdf")
 public class CASH extends AbstractAlgorithm<Clustering<Model>> implements ClusteringAlgorithm<Clustering<Model>> {
   /**
@@ -349,7 +354,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
         res.addCluster(c);
         noiseIDs.removeDBIDs(interval.getIDs());
         clusterIDs.addDBIDs(interval.getIDs());
-        processedIDs.addAll(interval.getIDs());
+        processedIDs.addDBIDs(interval.getIDs());
       }
 
       // Rebuild heap
@@ -372,13 +377,13 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
       if(dim == noiseDim) {
         Cluster<Model> c = new Cluster<Model>(noiseIDs, true, ClusterModel.CLUSTER);
         res.addCluster(c);
-        processedIDs.addAll(noiseIDs);
+        processedIDs.addDBIDs(noiseIDs);
       }
       else if(noiseIDs.size() >= minPts) {
         LinearEquationSystem les = runDerivator(fulldatabase, dim - 1, noiseIDs);
         Cluster<Model> c = new Cluster<Model>(noiseIDs, true, new LinearEquationModel(les));
         res.addCluster(c);
-        processedIDs.addAll(noiseIDs);
+        processedIDs.addDBIDs(noiseIDs);
       }
     }
 
@@ -498,9 +503,9 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
     proxy.addRelation(prep);
     
     // Project
-    for(DBID id : ids) {
-      ParameterizationFunction f = project(basis, relation.get(id));
-      prep.set(id, f);
+    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      ParameterizationFunction f = project(basis, relation.get(iter));
+      prep.set(iter, f);
     }
 
     if(logger.isDebugging()) {
@@ -521,7 +526,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
   private ParameterizationFunction project(Matrix basis, ParameterizationFunction f) {
     // Matrix m = new Matrix(new
     // double[][]{f.getPointCoordinates()}).times(basis);
-    Matrix m = f.getRowVector().times(basis);
+    Matrix m = f.getColumnVector().transposeTimes(basis);
     ParameterizationFunction f_t = new ParameterizationFunction(m.getColumnPackedCopy());
     return f_t;
   }
@@ -657,8 +662,8 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
 
     double d_min = Double.POSITIVE_INFINITY;
     double d_max = Double.NEGATIVE_INFINITY;
-    for(DBID id : relation.iterDBIDs()) {
-      ParameterizationFunction f = relation.get(id);
+    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      ParameterizationFunction f = relation.get(iditer);
       HyperBoundingBox minMax = f.determineAlphaMinMax(box);
       double f_min = f.function(SpatialUtil.getMin(minMax));
       double f_max = f.function(SpatialUtil.getMax(minMax));
@@ -704,11 +709,11 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
 
     ids.addDBIDs(interval.getIDs());
     // Search for nearby vectors in original database
-    for(DBID id : relation.iterDBIDs()) {
-      DoubleVector v = new DoubleVector(relation.get(id).getColumnVector().getArrayRef());
+    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      DoubleVector v = new DoubleVector(relation.get(iditer).getColumnVector().getArrayRef());
       DoubleDistance d = df.distance(v, centroid);
       if(d.compareTo(eps) < 0) {
-        ids.add(id);
+        ids.add(iditer);
       }
     }
 
@@ -730,15 +735,15 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
   private Database buildDerivatorDB(Relation<ParameterizationFunction> relation, CASHInterval interval) throws UnableToComplyException {
     DBIDs ids = interval.getIDs();
     ProxyDatabase proxy = new ProxyDatabase(ids);
-    int dim = relation.get(ids.iterator().next()).getDimensionality();
+    int dim = DatabaseUtil.dimensionality(relation);
     SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.class, dim, new DoubleVector(new double[dim]));
     MaterializedRelation<DoubleVector> prep = new MaterializedRelation<DoubleVector>(proxy, type, ids);
     proxy.addRelation(prep);
 
     // Project
-    for(DBID id : ids) {
-      DoubleVector v = new DoubleVector(relation.get(id).getColumnVector().getArrayRef());
-      prep.set(id, v);
+    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      DoubleVector v = new DoubleVector(relation.get(iter).getColumnVector().getArrayRef());
+      prep.set(iter, v);
     }
 
     if(logger.isDebugging()) {
@@ -795,15 +800,15 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    */
   private Database buildDerivatorDB(Relation<ParameterizationFunction> relation, DBIDs ids) throws UnableToComplyException {
     ProxyDatabase proxy = new ProxyDatabase(ids);
-    int dim = relation.get(ids.iterator().next()).getDimensionality();
+    int dim = DatabaseUtil.dimensionality(relation);
     SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.class, dim, new DoubleVector(new double[dim]));
     MaterializedRelation<DoubleVector> prep = new MaterializedRelation<DoubleVector>(proxy, type, ids);
     proxy.addRelation(prep);
 
     // Project
-    for(DBID id : ids) {
-      DoubleVector v = new DoubleVector(relation.get(id).getColumnVector().getArrayRef());
-      prep.set(id, v);
+    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      DoubleVector v = new DoubleVector(relation.get(iter).getColumnVector().getArrayRef());
+      prep.set(iter, v);
     }
 
     return proxy;
