@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.subspace;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -27,14 +27,12 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.SortedMap;
 import java.util.TreeMap;
 
 import de.lmu.ifi.dbs.elki.algorithm.AbstractAlgorithm;
-import de.lmu.ifi.dbs.elki.algorithm.clustering.ClusteringAlgorithm;
 import de.lmu.ifi.dbs.elki.algorithm.clustering.subspace.clique.CLIQUESubspace;
 import de.lmu.ifi.dbs.elki.algorithm.clustering.subspace.clique.CLIQUEUnit;
 import de.lmu.ifi.dbs.elki.data.Cluster;
@@ -46,6 +44,7 @@ import de.lmu.ifi.dbs.elki.data.model.SubspaceModel;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.logging.Logging;
@@ -97,7 +96,7 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 @Title("CLIQUE: Automatic Subspace Clustering of High Dimensional Data for Data Mining Applications")
 @Description("Grid-based algorithm to identify dense clusters in subspaces of maximum dimensionality.")
 @Reference(authors = "R. Agrawal, J. Gehrke, D. Gunopulos, P. Raghavan", title = "Automatic Subspace Clustering of High Dimensional Data for Data Mining Applications", booktitle = "Proc. SIGMOD Conference, Seattle, WA, 1998", url = "http://dx.doi.org/10.1145/276304.276314")
-public class CLIQUE<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clustering<SubspaceModel<V>>> implements ClusteringAlgorithm<Clustering<SubspaceModel<V>>> {
+public class CLIQUE<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clustering<SubspaceModel<V>>> implements SubspaceClusteringAlgorithm<SubspaceModel<V>> {
   /**
    * The logger for this class.
    */
@@ -299,8 +298,8 @@ public class CLIQUE<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
       minima[d] = Double.MAX_VALUE;
     }
     // update minima and maxima
-    for(Iterator<DBID> it = database.iterDBIDs(); it.hasNext();) {
-      V featureVector = database.get(it.next());
+    for(DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
+      V featureVector = database.get(it);
       updateMinMax(featureVector, minima, maxima);
     }
     for(int i = 0; i < maxima.length; i++) {
@@ -393,13 +392,15 @@ public class CLIQUE<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
 
     // identify dense units
     double total = database.size();
-    for(Iterator<DBID> it = database.iterDBIDs(); it.hasNext();) {
-      final DBID id = it.next();
-      V featureVector = database.get(id);
+    for(DBIDIter it = database.iterDBIDs(); it.valid();) {
+      V featureVector = database.get(it);
+      final DBID id = it.getDBID();
+      it.advance();
       for(CLIQUEUnit<V> unit : units) {
         unit.addFeatureVector(id, featureVector);
         // unit is a dense unit
-        if(!it.hasNext() && unit.selectivity(total) >= tau) {
+        // FIXME: why it.valid()?
+        if(!it.valid() && unit.selectivity(total) >= tau) {
           denseUnits.add(unit);
           // add the dense unit to its subspace
           int dim = unit.getIntervals().iterator().next().getDimension();
