@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.subspace;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -28,7 +28,6 @@ import java.util.BitSet;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Random;
@@ -39,13 +38,13 @@ import de.lmu.ifi.dbs.elki.data.Cluster;
 import de.lmu.ifi.dbs.elki.data.Clustering;
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.data.Subspace;
-import de.lmu.ifi.dbs.elki.data.model.Model;
 import de.lmu.ifi.dbs.elki.data.model.SubspaceModel;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
@@ -87,11 +86,11 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
  * 
  * @param <V> the type of NumberVector handled by this Algorithm
  */
+// TODO: optimize by creating much less objects
 @Title("PROCLUS: PROjected CLUStering")
 @Description("Algorithm to find subspace clusters in high dimensional spaces.")
 @Reference(authors = "C. C. Aggarwal, C. Procopiuc, J. L. Wolf, P. S. Yu, J. S. Park", title = "Fast Algorithms for Projected Clustering", booktitle = "Proc. ACM SIGMOD Int. Conf. on Management of Data (SIGMOD '99)", url = "http://dx.doi.org/10.1145/304181.304188")
-// TODO: make the generics reflect the SubspaceModel
-public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClustering<Clustering<Model>, V> {
+public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClustering<Clustering<SubspaceModel<V>>, V> implements SubspaceClusteringAlgorithm<SubspaceModel<V>> {
   /**
    * The logger for this class.
    */
@@ -141,8 +140,11 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
 
   /**
    * Performs the PROCLUS algorithm on the given database.
+   * 
+   * @param database Database to process
+   * @param relation Relation to process
    */
-  public Clustering<Model> run(Database database, Relation<V> relation) throws IllegalStateException {
+  public Clustering<SubspaceModel<V>> run(Database database, Relation<V> relation) {
     DistanceQuery<V, DoubleDistance> distFunc = this.getDistanceQuery(database);
     RangeQuery<V, DoubleDistance> rangeQuery = database.getRangeQuery(distFunc);
     final Random random = new Random();
@@ -193,6 +195,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
 
     IndefiniteProgress cprogress = logger.isVerbose() ? new IndefiniteProgress("Current number of clusters:", logger) : null;
 
+    // TODO: Use DataStore and Trove for performance
     Map<DBID, PROCLUSCluster> clusters = null;
     int loops = 0;
     while(loops < 10) {
@@ -229,9 +232,9 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
 
     // build result
     int numClusters = 1;
-    Clustering<Model> result = new Clustering<Model>("ProClus clustering", "proclus-clustering");
+    Clustering<SubspaceModel<V>> result = new Clustering<SubspaceModel<V>>("ProClus clustering", "proclus-clustering");
     for(PROCLUSCluster c : finalClusters) {
-      Cluster<Model> cluster = new Cluster<Model>(c.objectIDs);
+      Cluster<SubspaceModel<V>> cluster = new Cluster<SubspaceModel<V>>(c.objectIDs);
       cluster.setModel(new SubspaceModel<V>(new Subspace<V>(c.getDimensions()), c.centroid));
       cluster.setName("cluster_" + numClusters++);
 
@@ -262,7 +265,8 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
 
     // compute distances between each point in S and m_i
     Map<DBID, DistanceResultPair<DoubleDistance>> distances = new HashMap<DBID, DistanceResultPair<DoubleDistance>>();
-    for(DBID id : s) {
+    for(DBIDIter iter = s.iter(); iter.valid(); iter.advance()) {
+      DBID id = iter.getDBID();
       DoubleDistance dist = distFunc.distance(id, m_i);
       distances.put(id, new GenericDistanceResultPair<DoubleDistance>(dist, id));
     }
@@ -278,7 +282,8 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
       distances.remove(m_i);
 
       // compute distances of each point to closest medoid
-      for(DBID id : s) {
+      for(DBIDIter iter = s.iter(); iter.valid(); iter.advance()) {
+        DBID id = iter.getDBID();
         DoubleDistance dist_new = distFunc.distance(id, m_i);
         DoubleDistance dist_old = distances.get(id).getDistance();
 
@@ -323,12 +328,11 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    */
   private ModifiableDBIDs computeM_current(DBIDs m, DBIDs m_best, DBIDs m_bad, Random random) {
     ArrayModifiableDBIDs m_list = DBIDUtil.newArray(m);
-    for(DBID m_i : m_best) {
-      m_list.remove(m_i);
-    }
+    m_list.removeDBIDs(m_best);
 
     ModifiableDBIDs m_current = DBIDUtil.newHashSet();
-    for(DBID m_i : m_best) {
+    for(DBIDIter iter = m_best.iter(); iter.valid(); iter.advance()) {
+      DBID m_i = iter.getDBID();
       if(m_bad.contains(m_i)) {
         int currentSize = m_current.size();
         while(m_current.size() == currentSize) {
@@ -358,11 +362,13 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
   private Map<DBID, List<DistanceResultPair<DoubleDistance>>> getLocalities(DBIDs medoids, Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, RangeQuery<V, DoubleDistance> rangeQuery) {
     Map<DBID, List<DistanceResultPair<DoubleDistance>>> result = new HashMap<DBID, List<DistanceResultPair<DoubleDistance>>>();
 
-    for(DBID m : medoids) {
+    for(DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
+      DBID m = iter.getDBID();
       // determine minimum distance between current medoid m and any other
       // medoid m_i
       DoubleDistance minDist = null;
-      for(DBID m_i : medoids) {
+      for(DBIDIter iter2 = medoids.iter(); iter2.valid(); iter2.advance()) {
+        DBID m_i = iter2.getDBID();
         if(m_i == m) {
           continue;
         }
@@ -399,7 +405,8 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
     int dim = DatabaseUtil.dimensionality(database);
     Map<DBID, double[]> averageDistances = new HashMap<DBID, double[]>();
 
-    for(DBID m_i : medoids) {
+    for(DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
+      DBID m_i = iter.getDBID();
       V medoid_i = database.get(m_i);
       List<DistanceResultPair<DoubleDistance>> l_i = localities.get(m_i);
       double[] x_i = new double[dim];
@@ -417,7 +424,8 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
 
     Map<DBID, Set<Integer>> dimensionMap = new HashMap<DBID, Set<Integer>>();
     List<CTriple<Double, DBID, Integer>> z_ijs = new ArrayList<CTriple<Double, DBID, Integer>>();
-    for(DBID m_i : medoids) {
+    for(DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
+      DBID m_i = iter.getDBID();
       Set<Integer> dims_i = new HashSet<Integer>();
       dimensionMap.put(m_i, dims_i);
 
@@ -478,8 +486,8 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
     for(int i = 0; i < clusters.size(); i++) {
       PROCLUSCluster c_i = clusters.get(i);
       double[] x_i = new double[dim];
-      for(DBID id : c_i.objectIDs) {
-        V o = database.get(id);
+      for(DBIDIter iter = c_i.objectIDs.iter(); iter.valid(); iter.advance()) {
+        V o = database.get(iter);
         for(int d = 0; d < dim; d++) {
           x_i[d] += Math.abs(c_i.centroid.doubleValue(d + 1) - o.doubleValue(d + 1));
         }
@@ -560,8 +568,8 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
       clusterIDs.put(m_i, DBIDUtil.newHashSet());
     }
 
-    for(Iterator<DBID> it = database.iterDBIDs(); it.hasNext();) {
-      DBID p_id = it.next();
+    for(DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
+      DBID p_id = it.getDBID();
       V p = database.get(p_id);
       DistanceResultPair<DoubleDistance> minDist = null;
       for(DBID m_i : dimensions.keySet()) {
@@ -610,8 +618,8 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
       clusterIDs.put(i, DBIDUtil.newHashSet());
     }
 
-    for(Iterator<DBID> it = database.iterDBIDs(); it.hasNext();) {
-      DBID p_id = it.next();
+    for(DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
+      DBID p_id = it.getDBID();
       V p = database.get(p_id);
       Pair<DoubleDistance, Integer> minDist = null;
       for(int i = 0; i < dimensions.size(); i++) {
@@ -707,8 +715,8 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    */
   private double avgDistance(V centroid, DBIDs objectIDs, Relation<V> database, int dimension) {
     double avg = 0;
-    for(DBID objectID : objectIDs) {
-      V o = database.get(objectID);
+    for(DBIDIter iter = objectIDs.iter(); iter.valid(); iter.advance()) {
+      V o = database.get(iter);
       avg += Math.abs(centroid.doubleValue(dimension) - o.doubleValue(dimension));
     }
     return avg / objectIDs.size();
