@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -26,9 +26,6 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
-import java.util.LinkedList;
-import java.util.List;
-import java.util.PriorityQueue;
 
 import de.lmu.ifi.dbs.elki.algorithm.AbstractDistanceBasedAlgorithm;
 import de.lmu.ifi.dbs.elki.data.NumberVector;
@@ -38,14 +35,18 @@ import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.QueryUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
-import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
+import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
+import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
+import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
@@ -55,10 +56,11 @@ import de.lmu.ifi.dbs.elki.distance.similarityfunction.kernel.PolynomialKernelFu
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
 import de.lmu.ifi.dbs.elki.math.MeanVariance;
-import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.result.outlier.InvertedOutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.Heap;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
@@ -68,7 +70,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualCons
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
-import de.lmu.ifi.dbs.elki.utilities.pairs.FCPair;
+import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
 
 /**
  * Angle-Based Outlier Detection
@@ -177,29 +179,30 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
   public OutlierResult getRanking(Relation<V> relation, int k) {
     // Fix a static set of IDs
     staticids = DBIDUtil.newArray(relation.getDBIDs());
-    Collections.sort(staticids);
+    staticids.sort();
 
     KernelMatrix kernelMatrix = new KernelMatrix(primitiveKernelFunction, relation, staticids);
-    PriorityQueue<FCPair<Double, DBID>> pq = new PriorityQueue<FCPair<Double, DBID>>(relation.size(), Collections.reverseOrder());
+    Heap<DoubleObjPair<DBID>> pq = new Heap<DoubleObjPair<DBID>>(relation.size(), Collections.reverseOrder());
 
     // preprocess kNN neighborhoods
     assert (k == this.k);
     KNNQuery<V, DoubleDistance> knnQuery = QueryUtil.getKNNQuery(relation, getDistanceFunction(), k);
 
-    for(DBID objKey : relation.iterDBIDs()) {
-      MeanVariance s = new MeanVariance();
+    MeanVariance s = new MeanVariance();
+    for(DBIDIter objKey = relation.iterDBIDs(); objKey.valid(); objKey.advance()) {
+      s.reset();
 
       // System.out.println("Processing: " +objKey);
-      List<DistanceResultPair<DoubleDistance>> neighbors = knnQuery.getKNNForDBID(objKey, k);
+      KNNResult<DoubleDistance> neighbors = knnQuery.getKNNForDBID(objKey, k);
       Iterator<DistanceResultPair<DoubleDistance>> iter = neighbors.iterator();
       while(iter.hasNext()) {
-        DBID key1 = iter.next().getDBID();
+        DistanceResultPair<DoubleDistance> key1 = iter.next();
         // Iterator iter2 = data.keyIterator();
         Iterator<DistanceResultPair<DoubleDistance>> iter2 = neighbors.iterator();
         // PriorityQueue best = new PriorityQueue(false, k);
         while(iter2.hasNext()) {
-          DBID key2 = iter2.next().getDBID();
-          if(key2.equals(key1) || key1.equals(objKey) || key2.equals(objKey)) {
+          DistanceResultPair<DoubleDistance> key2 = iter2.next();
+          if(key2.sameDBID(key1) || key1.sameDBID(objKey) || key2.sameDBID(objKey)) {
             continue;
           }
           double nenner = calcDenominator(kernelMatrix, objKey, key1, key2);
@@ -214,14 +217,14 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
       }
       // Sample variance probably would be correct, however the numerical
       // instabilities can actually break ABOD here.
-      pq.add(new FCPair<Double, DBID>(s.getNaiveVariance(), objKey));
+      pq.add(new DoubleObjPair<DBID>(s.getNaiveVariance(), objKey.getDBID()));
     }
 
     DoubleMinMax minmaxabod = new DoubleMinMax();
-    WritableDataStore<Double> abodvalues = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC, Double.class);
-    for(FCPair<Double, DBID> pair : pq) {
-      abodvalues.put(pair.getSecond(), pair.getFirst());
-      minmaxabod.put(pair.getFirst());
+    WritableDoubleDataStore abodvalues = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC);
+    for(DoubleObjPair<DBID> pair : pq) {
+      abodvalues.putDouble(pair.getSecond(), pair.first);
+      minmaxabod.put(pair.first);
     }
     // Build result representation.
     Relation<Double> scoreResult = new MaterializedRelation<Double>("Angle-based Outlier Degree", "abod-outlier", TypeUtil.DOUBLE, abodvalues, relation.getDBIDs());
@@ -238,18 +241,20 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
    * @return result
    */
   public OutlierResult getFastRanking(Relation<V> relation, int k, int sampleSize) {
+    final DBIDs ids = relation.getDBIDs();
     // Fix a static set of IDs
-    staticids = DBIDUtil.newArray(relation.getDBIDs());
-    Collections.sort(staticids);
+    // TODO: add a DBIDUtil.ensureSorted?
+    staticids = DBIDUtil.newArray(ids);
+    staticids.sort();
 
     KernelMatrix kernelMatrix = new KernelMatrix(primitiveKernelFunction, relation, staticids);
 
-    PriorityQueue<FCPair<Double, DBID>> pq = new PriorityQueue<FCPair<Double, DBID>>(relation.size(), Collections.reverseOrder());
+    Heap<DoubleObjPair<DBID>> pq = new Heap<DoubleObjPair<DBID>>(relation.size(), Collections.reverseOrder());
     // get Candidate Ranking
-    for(DBID aKey : relation.iterDBIDs()) {
-      HashMap<DBID, Double> dists = new HashMap<DBID, Double>(relation.size());
+    for(DBIDIter aKey = relation.iterDBIDs(); aKey.valid(); aKey.advance()) {
+      WritableDoubleDataStore dists = DataStoreUtil.makeDoubleStorage(ids, DataStoreFactory.HINT_TEMP | DataStoreFactory.HINT_HOT);
       // determine kNearestNeighbors and pairwise distances
-      PriorityQueue<FCPair<Double, DBID>> nn;
+      Heap<DoubleObjPair<DBID>> nn;
       if(!useRNDSample) {
         nn = calcDistsandNN(relation, kernelMatrix, sampleSize, aKey, dists);
       }
@@ -259,7 +264,7 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
       }
 
       // get normalization
-      double[] counter = calcFastNormalization(aKey, dists);
+      double[] counter = calcFastNormalization(aKey, dists, staticids);
       // System.out.println(counter[0] + " " + counter2[0] + " " + counter[1] +
       // " " + counter2[1]);
       // umsetzen von Pq zu list
@@ -269,15 +274,16 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
       }
       // getFilter
       double var = getAbofFilter(kernelMatrix, aKey, dists, counter[1], counter[0], neighbors);
-      pq.add(new FCPair<Double, DBID>(var, aKey));
+      pq.add(new DoubleObjPair<DBID>(var, aKey.getDBID()));
       // System.out.println("prog "+(prog++));
     }
     // refine Candidates
-    PriorityQueue<FCPair<Double, DBID>> resqueue = new PriorityQueue<FCPair<Double, DBID>>(k);
+    Heap<DoubleObjPair<DBID>> resqueue = new Heap<DoubleObjPair<DBID>>(k);
     // System.out.println(pq.size() + " objects ordered into candidate list.");
     // int v = 0;
+    MeanVariance s = new MeanVariance();
     while(!pq.isEmpty()) {
-      if(resqueue.size() == k && pq.peek().getFirst() > resqueue.peek().getFirst()) {
+      if(resqueue.size() == k && pq.peek().first > resqueue.peek().first) {
         break;
       }
       // double approx = pq.peek().getFirst();
@@ -290,13 +296,13 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
       // + " worst result: " + Double.MAX_VALUE);
       // }
       // v++;
-      MeanVariance s = new MeanVariance();
-      for(DBID bKey : relation.iterDBIDs()) {
-        if(bKey.equals(aKey)) {
+      s.reset();
+      for(DBIDIter bKey = relation.iterDBIDs(); bKey.valid(); bKey.advance()) {
+        if(bKey.sameDBID(aKey)) {
           continue;
         }
-        for(DBID cKey : relation.iterDBIDs()) {
-          if(cKey.equals(aKey)) {
+        for(DBIDIter cKey = relation.iterDBIDs(); cKey.valid(); cKey.advance()) {
+          if(cKey.sameDBID(aKey)) {
             continue;
           }
           // double nenner = dists[y]*dists[z];
@@ -313,101 +319,72 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
       double var = s.getSampleVariance();
       // System.out.println(aKey+ " : " + approx +" " + var);
       if(resqueue.size() < k) {
-        resqueue.add(new FCPair<Double, DBID>(var, aKey));
+        resqueue.add(new DoubleObjPair<DBID>(var, aKey));
       }
       else {
-        if(resqueue.peek().getFirst() > var) {
+        if(resqueue.peek().first > var) {
           resqueue.remove();
-          resqueue.add(new FCPair<Double, DBID>(var, aKey));
+          resqueue.add(new DoubleObjPair<DBID>(var, aKey));
         }
       }
 
     }
     // System.out.println(v + " Punkte von " + data.size() + " verfeinert !!");
     DoubleMinMax minmaxabod = new DoubleMinMax();
-    WritableDataStore<Double> abodvalues = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC, Double.class);
-    for(FCPair<Double, DBID> pair : pq) {
-      abodvalues.put(pair.getSecond(), pair.getFirst());
-      minmaxabod.put(pair.getFirst());
+    WritableDoubleDataStore abodvalues = DataStoreUtil.makeDoubleStorage(ids, DataStoreFactory.HINT_STATIC);
+    for(DoubleObjPair<DBID> pair : pq) {
+      abodvalues.putDouble(pair.getSecond(), pair.first);
+      minmaxabod.put(pair.first);
     }
     // Build result representation.
-    Relation<Double> scoreResult = new MaterializedRelation<Double>("Angle-based Outlier Detection", "abod-outlier", TypeUtil.DOUBLE, abodvalues, relation.getDBIDs());
+    Relation<Double> scoreResult = new MaterializedRelation<Double>("Angle-based Outlier Detection", "abod-outlier", TypeUtil.DOUBLE, abodvalues, ids);
     OutlierScoreMeta scoreMeta = new InvertedOutlierScoreMeta(minmaxabod.getMin(), minmaxabod.getMax(), 0.0, Double.POSITIVE_INFINITY);
     return new OutlierResult(scoreMeta, scoreResult);
   }
 
-  // TODO: remove?
-  @SuppressWarnings("unused")
-  private double[] calcNormalization(Integer xKey, HashMap<Integer, Double> dists) {
-    double[] result = new double[2];
-    for(Integer yKey : dists.keySet()) {
-      if(yKey.equals(xKey)) {
-        continue;
-      }
-      for(Integer zKey : dists.keySet()) {
-        if(zKey <= yKey) {
-          continue;
-        }
-        if(zKey.equals(xKey)) {
-          continue;
-        }
-        if(dists.get(yKey) != 0 && dists.get(zKey) != 0) {
-          double sqr = Math.sqrt(dists.get(yKey) * dists.get(zKey));
-          result[0] += 1 / sqr;
-          result[1] += 1 / (dists.get(yKey) * dists.get(zKey) * sqr);
-        }
-      }
-    }
-    return result;
-  }
-
-  private double[] calcFastNormalization(DBID x, HashMap<DBID, Double> dists) {
+  private double[] calcFastNormalization(DBIDRef x, WritableDoubleDataStore dists, DBIDs ids) {
     double[] result = new double[2];
 
     double sum = 0;
     double sumF = 0;
-    for(DBID yKey : dists.keySet()) {
-      if(dists.get(yKey) != 0) {
-        double tmp = 1 / Math.sqrt(dists.get(yKey));
+    for (DBIDIter yKey = ids.iter(); yKey.valid(); yKey.advance()) {
+      if(dists.doubleValue(yKey) != 0) {
+        double tmp = 1 / Math.sqrt(dists.doubleValue(yKey));
         sum += tmp;
-        sumF += (1 / dists.get(yKey)) * tmp;
+        sumF += (1 / dists.doubleValue(yKey)) * tmp;
       }
     }
     double sofar = 0;
     double sofarF = 0;
-    for(DBID zKey : dists.keySet()) {
-      if(dists.get(zKey) != 0) {
-        double tmp = 1 / Math.sqrt(dists.get(zKey));
+    for (DBIDIter zKey = ids.iter(); zKey.valid(); zKey.advance()) {
+      if(dists.doubleValue(zKey) != 0) {
+        double tmp = 1 / Math.sqrt(dists.doubleValue(zKey));
         sofar += tmp;
         double rest = sum - sofar;
         result[0] += tmp * rest;
 
-        sofarF += (1 / dists.get(zKey)) * tmp;
+        sofarF += (1 / dists.doubleValue(zKey)) * tmp;
         double restF = sumF - sofarF;
-        result[1] += (1 / dists.get(zKey)) * tmp * restF;
+        result[1] += (1 / dists.doubleValue(zKey)) * tmp * restF;
       }
     }
     return result;
   }
 
-  private double getAbofFilter(KernelMatrix kernelMatrix, DBID aKey, HashMap<DBID, Double> dists, double fulCounter, double counter, DBIDs neighbors) {
+  private double getAbofFilter(KernelMatrix kernelMatrix, DBIDRef aKey, WritableDoubleDataStore dists, double fulCounter, double counter, DBIDs neighbors) {
     double sum = 0.0;
     double sqrSum = 0.0;
     double partCounter = 0;
-    Iterator<DBID> iter = neighbors.iterator();
-    while(iter.hasNext()) {
-      DBID bKey = iter.next();
-      if(bKey.equals(aKey)) {
+    for(DBIDIter bKey = neighbors.iter(); bKey.valid(); bKey.advance()) {
+      if(bKey.sameDBID(aKey)) {
         continue;
       }
-      Iterator<DBID> iter2 = neighbors.iterator();
-      while(iter2.hasNext()) {
-        DBID cKey = iter2.next();
-        if(cKey.equals(aKey)) {
+      for(DBIDIter cKey = neighbors.iter(); cKey.valid(); cKey.advance()) {
+        if(cKey.sameDBID(aKey)) {
           continue;
         }
-        if(bKey.compareTo(cKey) > 0) {
-          double nenner = dists.get(bKey).doubleValue() * dists.get(cKey).doubleValue();
+        if(bKey.compareDBID(cKey) > 0) {
+          double nenner = dists.doubleValue(bKey) * dists.doubleValue(cKey);
           if(nenner != 0) {
             double tmp = calcNumerator(kernelMatrix, aKey, bKey, cKey) / nenner;
             double sqrtNenner = Math.sqrt(nenner);
@@ -431,59 +408,59 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
    * @param bKey
    * @return cosinus value
    */
-  private double calcCos(KernelMatrix kernelMatrix, DBID aKey, DBID bKey) {
+  private double calcCos(KernelMatrix kernelMatrix, DBIDRef aKey, DBIDRef bKey) {
     final int ai = mapDBID(aKey);
     final int bi = mapDBID(bKey);
     return kernelMatrix.getDistance(ai, ai) + kernelMatrix.getDistance(bi, bi) - 2 * kernelMatrix.getDistance(ai, bi);
   }
 
-  private int mapDBID(DBID aKey) {
+  private int mapDBID(DBIDRef aKey) {
     // TODO: this is not the most efficient...
-    int off = Collections.binarySearch(staticids, aKey);
+    int off = staticids.binarySearch(aKey);
     if(off < 0) {
       throw new AbortException("Did not find id " + aKey.toString() + " in staticids. " + staticids.contains(aKey));
     }
     return off + 1;
   }
 
-  private double calcDenominator(KernelMatrix kernelMatrix, DBID aKey, DBID bKey, DBID cKey) {
+  private double calcDenominator(KernelMatrix kernelMatrix, DBIDRef aKey, DBIDRef bKey, DBIDRef cKey) {
     return calcCos(kernelMatrix, aKey, bKey) * calcCos(kernelMatrix, aKey, cKey);
   }
 
-  private double calcNumerator(KernelMatrix kernelMatrix, DBID aKey, DBID bKey, DBID cKey) {
+  private double calcNumerator(KernelMatrix kernelMatrix, DBIDRef aKey, DBIDRef bKey, DBIDRef cKey) {
     final int ai = mapDBID(aKey);
     final int bi = mapDBID(bKey);
     final int ci = mapDBID(cKey);
     return (kernelMatrix.getDistance(ai, ai) + kernelMatrix.getDistance(bi, ci) - kernelMatrix.getDistance(ai, ci) - kernelMatrix.getDistance(ai, bi));
   }
 
-  private PriorityQueue<FCPair<Double, DBID>> calcDistsandNN(Relation<V> data, KernelMatrix kernelMatrix, int sampleSize, DBID aKey, HashMap<DBID, Double> dists) {
-    PriorityQueue<FCPair<Double, DBID>> nn = new PriorityQueue<FCPair<Double, DBID>>(sampleSize);
-    for(DBID bKey : data.iterDBIDs()) {
+  private Heap<DoubleObjPair<DBID>> calcDistsandNN(Relation<V> data, KernelMatrix kernelMatrix, int sampleSize, DBIDRef aKey, WritableDoubleDataStore dists) {
+    Heap<DoubleObjPair<DBID>> nn = new Heap<DoubleObjPair<DBID>>(sampleSize);
+    for(DBIDIter bKey = data.iterDBIDs(); bKey.valid(); bKey.advance()) {
       double val = calcCos(kernelMatrix, aKey, bKey);
-      dists.put(bKey, val);
+      dists.putDouble(bKey, val);
       if(nn.size() < sampleSize) {
-        nn.add(new FCPair<Double, DBID>(val, bKey));
+        nn.add(new DoubleObjPair<DBID>(val, bKey.getDBID()));
       }
       else {
-        if(val < nn.peek().getFirst()) {
+        if(val < nn.peek().first) {
           nn.remove();
-          nn.add(new FCPair<Double, DBID>(val, bKey));
+          nn.add(new DoubleObjPair<DBID>(val, bKey.getDBID()));
         }
       }
     }
     return nn;
   }
 
-  private PriorityQueue<FCPair<Double, DBID>> calcDistsandRNDSample(Relation<V> data, KernelMatrix kernelMatrix, int sampleSize, DBID aKey, HashMap<DBID, Double> dists) {
-    PriorityQueue<FCPair<Double, DBID>> nn = new PriorityQueue<FCPair<Double, DBID>>(sampleSize);
+  private Heap<DoubleObjPair<DBID>> calcDistsandRNDSample(Relation<V> data, KernelMatrix kernelMatrix, int sampleSize, DBIDRef aKey, WritableDoubleDataStore dists) {
+    Heap<DoubleObjPair<DBID>> nn = new Heap<DoubleObjPair<DBID>>(sampleSize);
     int step = (int) ((double) data.size() / (double) sampleSize);
     int counter = 0;
-    for(DBID bKey : data.iterDBIDs()) {
+    for(DBIDIter bKey = data.iterDBIDs(); bKey.valid(); bKey.advance()) {
       double val = calcCos(kernelMatrix, aKey, bKey);
-      dists.put(bKey, val);
+      dists.putDouble(bKey, val);
       if(counter % step == 0) {
-        nn.add(new FCPair<Double, DBID>(val, bKey));
+        nn.add(new DoubleObjPair<DBID>(val, bKey.getDBID()));
       }
       counter++;
     }
@@ -499,27 +476,24 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
   public void getExplanations(Relation<V> data) {
     KernelMatrix kernelMatrix = new KernelMatrix(primitiveKernelFunction, data, staticids);
     // PQ for Outlier Ranking
-    PriorityQueue<FCPair<Double, DBID>> pq = new PriorityQueue<FCPair<Double, DBID>>(data.size(), Collections.reverseOrder());
-    HashMap<DBID, LinkedList<DBID>> explaintab = new HashMap<DBID, LinkedList<DBID>>();
+    Heap<DoubleObjPair<DBID>> pq = new Heap<DoubleObjPair<DBID>>(data.size(), Collections.reverseOrder());
+    HashMap<DBID, DBIDs> explaintab = new HashMap<DBID, DBIDs>();
     // test all objects
-    for(DBID objKey : data.iterDBIDs()) {
-      MeanVariance s = new MeanVariance();
+    MeanVariance s = new MeanVariance(), s2 = new MeanVariance();
+    for(DBIDIter objKey = data.iterDBIDs(); objKey.valid(); objKey.advance()) {
+      s.reset();
       // Queue for the best explanation
-      PriorityQueue<FCPair<Double, DBID>> explain = new PriorityQueue<FCPair<Double, DBID>>();
+      Heap<DoubleObjPair<DBID>> explain = new Heap<DoubleObjPair<DBID>>();
       // determine Object
       // for each pair of other objects
-      Iterator<DBID> iter = data.iterDBIDs();
+      for (DBIDIter key1 = data.iterDBIDs(); key1.valid(); key1.advance()) {
       // Collect Explanation Vectors
-      while(iter.hasNext()) {
-        MeanVariance s2 = new MeanVariance();
-        DBID key1 = iter.next();
-        Iterator<DBID> iter2 = data.iterDBIDs();
-        if(objKey.equals(key1)) {
+        s2.reset();
+        if(objKey.sameDBID(key1)) {
           continue;
         }
-        while(iter2.hasNext()) {
-          DBID key2 = iter2.next();
-          if(key2.equals(key1) || objKey.equals(key2)) {
+        for (DBIDIter key2 = data.iterDBIDs(); key2.valid(); key2.advance()) {
+          if(key2.sameDBID(key1) || objKey.sameDBID(key2)) {
             continue;
           }
           double nenner = calcDenominator(kernelMatrix, objKey, key1, key2);
@@ -529,22 +503,22 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
             s2.put(tmp, 1 / sqr);
           }
         }
-        explain.add(new FCPair<Double, DBID>(s2.getSampleVariance(), key1));
+        explain.add(new DoubleObjPair<DBID>(s2.getSampleVariance(), key1.getDBID()));
         s.put(s2);
       }
       // build variance of the observed vectors
-      pq.add(new FCPair<Double, DBID>(s.getSampleVariance(), objKey));
+      pq.add(new DoubleObjPair<DBID>(s.getSampleVariance(), objKey.getDBID()));
       //
-      LinkedList<DBID> expList = new LinkedList<DBID>();
+      ModifiableDBIDs expList = DBIDUtil.newArray();
       expList.add(explain.remove().getSecond());
       while(!explain.isEmpty()) {
         DBID nextKey = explain.remove().getSecond();
-        if(nextKey.equals(objKey)) {
+        if(nextKey.sameDBID(objKey)) {
           continue;
         }
         double max = Double.MIN_VALUE;
-        for(DBID exp : expList) {
-          if(exp.equals(objKey) || nextKey.equals(exp)) {
+        for(DBIDIter exp = expList.iter(); exp.valid(); exp.advance()) {
+          if(exp.sameDBID(objKey) || nextKey.sameDBID(exp)) {
             continue;
           }
           double nenner = Math.sqrt(calcCos(kernelMatrix, objKey, nextKey)) * Math.sqrt(calcCos(kernelMatrix, objKey, exp));
@@ -555,7 +529,7 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
           expList.add(nextKey);
         }
       }
-      explaintab.put(objKey, expList);
+      explaintab.put(objKey.getDBID(), expList);
     }
     System.out.println("--------------------------------------------");
     System.out.println("Result: ABOD");
@@ -564,26 +538,25 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
       if(count > 10) {
         break;
       }
-      double factor = pq.peek().getFirst();
+      double factor = pq.peek().first;
       DBID key = pq.remove().getSecond();
       System.out.print(data.get(key) + " ");
       System.out.println(count + " Factor=" + factor + " " + key);
-      LinkedList<DBID> expList = explaintab.get(key);
+      DBIDs expList = explaintab.get(key);
       generateExplanation(data, key, expList);
       count++;
     }
     System.out.println("--------------------------------------------");
   }
 
-  private void generateExplanation(Relation<V> data, DBID key, LinkedList<DBID> expList) {
-    V vect1 = data.get(key);
-    Iterator<DBID> iter = expList.iterator();
-    while(iter.hasNext()) {
+  private void generateExplanation(Relation<V> data, DBID key, DBIDs expList) {
+    Vector vect1 = data.get(key).getColumnVector();
+    for(DBIDIter iter = expList.iter(); iter.valid(); iter.advance()) {
       System.out.println("Outlier: " + vect1);
-      V exp = data.get(iter.next());
+      Vector exp = data.get(iter).getColumnVector();
       System.out.println("Most common neighbor: " + exp);
       // determine difference Vector
-      V vals = exp.minus(vect1);
+      Vector vals = exp.minus(vect1);
       System.out.println(vals);
       // System.out.println(new FeatureVector(
       // "Diff-"+vect1.getPrimaryKey(),vals ));
