@@ -1,26 +1,27 @@
 package de.lmu.ifi.dbs.elki.algorithm.outlier;
+
 /*
-This file is part of ELKI:
-Environment for Developing KDD-Applications Supported by Index-Structures
+ This file is part of ELKI:
+ Environment for Developing KDD-Applications Supported by Index-Structures
 
-Copyright (C) 2011
-Ludwig-Maximilians-Universität München
-Lehr- und Forschungseinheit für Datenbanksysteme
-ELKI Development Team
+ Copyright (C) 2012
+ Ludwig-Maximilians-Universität München
+ Lehr- und Forschungseinheit für Datenbanksysteme
+ ELKI Development Team
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as published by
+ the Free Software Foundation, either version 3 of the License, or
+ (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU Affero General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -35,13 +36,14 @@ import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
-import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
+import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
-import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.result.outlier.InvertedOutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
@@ -65,8 +67,7 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
  * detect outliers for high dimensional data.
  * <p>
  * Reference: <br />
- * Outlier detection for high dimensional data Outlier detection for high
- * dimensional data <br />
+ * Outlier detection for high dimensional data<br />
  * C.C. Aggarwal, P. S. Yu <br />
  * Proceedings of the 2001 ACM SIGMOD international conference on Management of
  * data 2001, Santa Barbara, California, United States
@@ -139,31 +140,32 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
    * @param database Database
    * @param relation Relation
    * @return Result
-   * @throws IllegalStateException
    */
-  public OutlierResult run(Database database, Relation<V> relation) throws IllegalStateException {
+  public OutlierResult run(Database database, Relation<V> relation) {
     final int dbsize = relation.size();
     ArrayList<ArrayList<DBIDs>> ranges = buildRanges(relation);
 
     Collection<Individuum> individuums = (new EvolutionarySearch(relation, ranges, m, seed)).run();
 
-    WritableDataStore<Double> outlierScore = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC, Double.class);
+    WritableDoubleDataStore outlierScore = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC);
     for(Individuum ind : individuums) {
       DBIDs ids = computeSubspaceForGene(ind.getGene(), ranges);
       double sparsityC = sparsity(ids.size(), dbsize, k);
-      for(DBID id : ids) {
-        Double prev = outlierScore.get(id);
-        if(prev == null || sparsityC < prev) {
-          outlierScore.put(id, sparsityC);
+      for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+        DBID id = iter.getDBID();
+        double prev = outlierScore.doubleValue(id);
+        if(Double.isNaN(prev) || sparsityC < prev) {
+          outlierScore.putDouble(id, sparsityC);
         }
       }
     }
 
     DoubleMinMax minmax = new DoubleMinMax();
-    for(DBID id : relation.iterDBIDs()) {
-      Double val = outlierScore.get(id);
-      if(val == null) {
-        outlierScore.put(id, 0.0);
+    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      DBID id  = iditer.getDBID();
+      double val = outlierScore.doubleValue(id);
+      if(Double.isNaN(val)) {
+        outlierScore.putDouble(id, 0.0);
         val = 0.0;
       }
       minmax.put(val);
@@ -224,9 +226,10 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
       this.m = m;
       this.dbsize = database.size();
       this.dim = DatabaseUtil.dimensionality(database);
-      if (seed != null) {
+      if(seed != null) {
         this.random = new Random(seed);
-      } else {
+      }
+      else {
         this.random = new Random();
       }
     }
@@ -274,7 +277,6 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
 
     /**
      * check the termination criterion
-     * 
      */
     private boolean checkConvergence(Collection<Individuum> pop) {
       if(pop.size() == 0) {
@@ -684,14 +686,14 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
 
   /**
    * Parameterization class.
-   *
+   * 
    * @author Erich Schubert
-   *
+   * 
    * @apiviz.exclude
    */
   public static class Parameterizer<V extends NumberVector<?, ?>> extends AbstractAggarwalYuOutlier.Parameterizer {
     protected int m = 0;
-    
+
     protected Long seed = null;
 
     @Override
