@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,8 +23,6 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.List;
-
 import de.lmu.ifi.dbs.elki.algorithm.AbstractAlgorithm;
 import de.lmu.ifi.dbs.elki.algorithm.AbstractDistanceBasedAlgorithm;
 import de.lmu.ifi.dbs.elki.data.type.CombinedTypeInformation;
@@ -34,13 +32,14 @@ import de.lmu.ifi.dbs.elki.database.QueryUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStore;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
-import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.query.DatabaseQuery;
 import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
+import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.query.knn.PreprocessorKNNQuery;
 import de.lmu.ifi.dbs.elki.database.query.rknn.RKNNQuery;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
@@ -52,6 +51,7 @@ import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.logging.progress.StepProgress;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
+import de.lmu.ifi.dbs.elki.math.Mean;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.result.outlier.QuotientOutlierScoreMeta;
@@ -169,9 +169,24 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
     this.reachabilityDistanceFunction = reachabilityDistanceFunction;
   }
 
+  /**
+   * Constructor.
+   * 
+   * @param k the value of k
+   * @param distanceFunction the distance function
+   * 
+   *        Uses the same distance function for neighborhood computation and
+   *        reachability distance (standard as in the original publication),
+   *        same as {@link #LOF(int, DistanceFunction, DistanceFunction)
+   *        LOF(int, distanceFunction, distanceFunction)}.
+   */
+  public LOF(int k, DistanceFunction<? super O, D> distanceFunction) {
+    this(k, distanceFunction, distanceFunction);
+  }
+
   /**
    * Performs the Generalized LOF_SCORE algorithm on the given database by
-   * calling {@code #doRunInTime(Database)}.
+   * calling {@link #doRunInTime}.
    * 
    * @param relation Data to process
    */
@@ -180,7 +195,7 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
     Pair<KNNQuery<O, D>, KNNQuery<O, D>> pair = getKNNQueries(relation, stepprog);
     KNNQuery<O, D> kNNRefer = pair.getFirst();
     KNNQuery<O, D> kNNReach = pair.getSecond();
-    return doRunInTime(kNNRefer, kNNReach, stepprog).getResult();
+    return doRunInTime(relation.getDBIDs(), kNNRefer, kNNReach, stepprog).getResult();
   }
 
   /**
@@ -227,11 +242,14 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
    * returns a {@link LOF.LOFResult} encapsulating information that may be
    * needed by an OnlineLOF algorithm.
    * 
+   * @param ids Object ids
    * @param kNNRefer the kNN query w.r.t. reference neighborhood distance
    *        function
    * @param kNNReach the kNN query w.r.t. reachability distance function
+   * @param stepprog Progress logger
+   * @return LOF result
    */
-  protected LOFResult<O, D> doRunInTime(KNNQuery<O, D> kNNRefer, KNNQuery<O, D> kNNReach, StepProgress stepprog) throws IllegalStateException {
+  protected LOFResult<O, D> doRunInTime(DBIDs ids, KNNQuery<O, D> kNNRefer, KNNQuery<O, D> kNNReach, StepProgress stepprog) {
     // Assert we got something
     if(kNNRefer == null) {
       throw new AbortException("No kNN queries supported by database for reference neighborhood distance function.");
@@ -244,14 +262,14 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
     if(stepprog != null) {
       stepprog.beginStep(2, "Computing LRDs.", logger);
     }
-    WritableDataStore<Double> lrds = computeLRDs(kNNReach.getRelation().getDBIDs(), kNNReach);
+    WritableDoubleDataStore lrds = computeLRDs(ids, kNNReach);
 
     // compute LOF_SCORE of each db object
     if(stepprog != null) {
       stepprog.beginStep(3, "Computing LOFs.", logger);
     }
-    Pair<WritableDataStore<Double>, DoubleMinMax> lofsAndMax = computeLOFs(kNNRefer.getRelation().getDBIDs(), lrds, kNNRefer);
-    WritableDataStore<Double> lofs = lofsAndMax.getFirst();
+    Pair<WritableDoubleDataStore, DoubleMinMax> lofsAndMax = computeLOFs(ids, lrds, kNNRefer);
+    WritableDoubleDataStore lofs = lofsAndMax.getFirst();
     // track the maximum value for normalization.
     DoubleMinMax lofminmax = lofsAndMax.getSecond();
 
@@ -260,7 +278,7 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
     }
 
     // Build result representation.
-    Relation<Double> scoreResult = new MaterializedRelation<Double>("Local Outlier Factor", "lof-outlier", TypeUtil.DOUBLE, lofs, kNNRefer.getRelation().getDBIDs());
+    Relation<Double> scoreResult = new MaterializedRelation<Double>("Local Outlier Factor", "lof-outlier", TypeUtil.DOUBLE, lofs, ids);
     OutlierScoreMeta scoreMeta = new QuotientOutlierScoreMeta(lofminmax.getMin(), lofminmax.getMax(), 0.0, Double.POSITIVE_INFINITY, 1.0);
     OutlierResult result = new OutlierResult(scoreMeta, scoreResult);
 
@@ -275,22 +293,22 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
    *        reachability distance
    * @return the LRDs of the objects
    */
-  protected WritableDataStore<Double> computeLRDs(DBIDs ids, KNNQuery<O, D> knnReach) {
-    WritableDataStore<Double> lrds = DataStoreUtil.makeStorage(ids, DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_TEMP, Double.class);
+  protected WritableDoubleDataStore computeLRDs(DBIDs ids, KNNQuery<O, D> knnReach) {
+    WritableDoubleDataStore lrds = DataStoreUtil.makeDoubleStorage(ids, DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_TEMP);
     FiniteProgress lrdsProgress = logger.isVerbose() ? new FiniteProgress("LRD", ids.size(), logger) : null;
-    for(DBID id : ids) {
-      double sum = 0;
-      List<DistanceResultPair<D>> neighbors = knnReach.getKNNForDBID(id, k);
-      int nsize = neighbors.size() - (objectIsInKNN ? 0 : 1);
+    Mean mean = new Mean();
+    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      mean.reset();
+      KNNResult<D> neighbors = knnReach.getKNNForDBID(iter, k);
       for(DistanceResultPair<D> neighbor : neighbors) {
-        if(objectIsInKNN || !neighbor.getDBID().equals(id)) {
-          List<DistanceResultPair<D>> neighborsNeighbors = knnReach.getKNNForDBID(neighbor.getDBID(), k);
-          sum += Math.max(neighbor.getDistance().doubleValue(), neighborsNeighbors.get(neighborsNeighbors.size() - 1).getDistance().doubleValue());
+        if(objectIsInKNN || !neighbor.sameDBID(iter)) {
+          KNNResult<D> neighborsNeighbors = knnReach.getKNNForDBID(neighbor, k);
+          mean.put(Math.max(neighbor.getDistance().doubleValue(), neighborsNeighbors.getKNNDistance().doubleValue()));
         }
       }
       // Avoid division by 0
-      Double lrd = (sum > 0) ? nsize / sum : 0.0;
-      lrds.put(id, lrd);
+      final double lrd = (mean.getCount() > 0) ? 1 / mean.getMean() : 0.0;
+      lrds.putDouble(iter, lrd);
       if(lrdsProgress != null) {
         lrdsProgress.incrementProcessed(logger);
       }
@@ -310,32 +328,31 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
    *        reference distance
    * @return the LOFs of the objects and the maximum LOF
    */
-  protected Pair<WritableDataStore<Double>, DoubleMinMax> computeLOFs(DBIDs ids, DataStore<Double> lrds, KNNQuery<O, D> knnRefer) {
-    WritableDataStore<Double> lofs = DataStoreUtil.makeStorage(ids, DataStoreFactory.HINT_STATIC, Double.class);
+  protected Pair<WritableDoubleDataStore, DoubleMinMax> computeLOFs(DBIDs ids, DataStore<Double> lrds, KNNQuery<O, D> knnRefer) {
+    WritableDoubleDataStore lofs = DataStoreUtil.makeDoubleStorage(ids, DataStoreFactory.HINT_STATIC);
     // track the maximum value for normalization.
     DoubleMinMax lofminmax = new DoubleMinMax();
 
     FiniteProgress progressLOFs = logger.isVerbose() ? new FiniteProgress("LOF_SCORE for objects", ids.size(), logger) : null;
-    for(DBID id : ids) {
-      double lrdp = lrds.get(id);
-      final Double lof;
+    Mean mean = new Mean();
+    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      double lrdp = lrds.get(iter);
+      final double lof;
       if(lrdp > 0) {
-        List<DistanceResultPair<D>> neighbors = knnRefer.getKNNForDBID(id, k);
-        int nsize = neighbors.size() - (objectIsInKNN ? 0 : 1);
-        // skip the point itself
-        // neighbors.remove(0);
-        double sum = 0;
+        final KNNResult<D> neighbors = knnRefer.getKNNForDBID(iter, k);
+        mean.reset();
         for(DistanceResultPair<D> neighbor : neighbors) {
-          if(objectIsInKNN || !neighbor.getDBID().equals(id)) {
-            sum += lrds.get(neighbor.getDBID());
+          // skip the point itself
+          if(objectIsInKNN || !neighbor.sameDBID(iter)) {
+            mean.put(lrds.get(neighbor));
           }
         }
-        lof = (sum / nsize) / lrdp;
+        lof = mean.getMean() / lrdp;
       }
       else {
         lof = 1.0;
       }
-      lofs.put(id, lof);
+      lofs.putDouble(iter, lof);
       // update minimum and maximum
       lofminmax.put(lof);
 
@@ -346,7 +363,7 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
     if(progressLOFs != null) {
       progressLOFs.ensureCompleted(logger);
     }
-    return new Pair<WritableDataStore<Double>, DoubleMinMax>(lofs, lofminmax);
+    return new Pair<WritableDoubleDataStore, DoubleMinMax>(lofs, lofminmax);
   }
 
   @Override
@@ -399,12 +416,12 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
     /**
      * The LRD values of the objects.
      */
-    private final WritableDataStore<Double> lrds;
+    private final WritableDoubleDataStore lrds;
 
     /**
      * The LOF values of the objects.
      */
-    private final WritableDataStore<Double> lofs;
+    private final WritableDoubleDataStore lofs;
 
     /**
      * Encapsulates information generated during a run of the {@link LOF}
@@ -416,7 +433,7 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
      * @param lrds the LRD values of the objects
      * @param lofs the LOF values of the objects
      */
-    public LOFResult(OutlierResult result, KNNQuery<O, D> kNNRefer, KNNQuery<O, D> kNNReach, WritableDataStore<Double> lrds, WritableDataStore<Double> lofs) {
+    public LOFResult(OutlierResult result, KNNQuery<O, D> kNNRefer, KNNQuery<O, D> kNNReach, WritableDoubleDataStore lrds, WritableDoubleDataStore lofs) {
       this.result = result;
       this.kNNRefer = kNNRefer;
       this.kNNReach = kNNReach;
@@ -441,14 +458,14 @@ public class LOF<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<Ou
     /**
      * @return the LRD values of the objects
      */
-    public WritableDataStore<Double> getLrds() {
+    public WritableDoubleDataStore getLrds() {
       return lrds;
     }
 
     /**
      * @return the LOF values of the objects
      */
-    public WritableDataStore<Double> getLofs() {
+    public WritableDoubleDataStore getLofs() {
       return lofs;
     }
 
