@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -38,5 +38,5 @@ public interface OutlierAlgorithm extends Algorithm {
   // Note: usually you won't override this method directly, but instead
   // Use the magic in AbstractAlgorithm and just implement a run method for your input data
   @Override
-  OutlierResult run(Database database) throws IllegalStateException;
+  OutlierResult run(Database database);
 }
\ No newline at end of file
