@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.algorithm.statistics;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -31,7 +31,7 @@ import java.util.Random;
 import java.util.TreeSet;
 
 import de.lmu.ifi.dbs.elki.algorithm.AbstractDistanceBasedAlgorithm;
-import de.lmu.ifi.dbs.elki.algorithm.clustering.trivial.ByLabelClustering;
+import de.lmu.ifi.dbs.elki.algorithm.clustering.trivial.ByLabelOrAllInOneClustering;
 import de.lmu.ifi.dbs.elki.data.Cluster;
 import de.lmu.ifi.dbs.elki.data.DoubleVector;
 import de.lmu.ifi.dbs.elki.data.model.Model;
@@ -40,6 +40,7 @@ import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
@@ -49,10 +50,10 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.logging.progress.StepProgress;
-import de.lmu.ifi.dbs.elki.math.AggregatingHistogram;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
-import de.lmu.ifi.dbs.elki.math.FlexiHistogram;
 import de.lmu.ifi.dbs.elki.math.MeanVariance;
+import de.lmu.ifi.dbs.elki.math.histograms.AggregatingHistogram;
+import de.lmu.ifi.dbs.elki.math.histograms.FlexiHistogram;
 import de.lmu.ifi.dbs.elki.result.CollectionResult;
 import de.lmu.ifi.dbs.elki.result.HistogramResult;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
@@ -65,7 +66,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameteriz
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Flag;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Parameter;
-import de.lmu.ifi.dbs.elki.utilities.pairs.FCPair;
+import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
 import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 
 /**
@@ -130,11 +131,8 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     this.sampling = sampling;
   }
 
-  /**
-   * Iterates over all points in the database.
-   */
   @Override
-  public HistogramResult<DoubleVector> run(Database database) throws IllegalStateException {
+  public HistogramResult<DoubleVector> run(Database database) {
     final Relation<O> relation = database.getRelation(getInputTypeRestriction()[0]);
     final DistanceQuery<O, D> distFunc = database.getDistanceQuery(relation, getDistanceFunction());
 
@@ -144,7 +142,7 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     DoubleMinMax gminmax = new DoubleMinMax();
 
     // Cluster by labels
-    Collection<Cluster<Model>> split = (new ByLabelClustering()).run(database).getAllClusters();
+    Collection<Cluster<Model>> split = (new ByLabelOrAllInOneClustering()).run(database).getAllClusters();
 
     // global in-cluster min/max
     DoubleMinMax giminmax = new DoubleMinMax();
@@ -183,12 +181,14 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     final Pair<Long, Long> incFirst = new Pair<Long, Long>(1L, 0L);
     final Pair<Long, Long> incSecond = new Pair<Long, Long>(0L, 1L);
     for(Cluster<?> c1 : split) {
-      for(DBID id1 : c1.getIDs()) {
+      for(DBIDIter iter = c1.getIDs().iter(); iter.valid(); iter.advance()) {
+        DBID id1 = iter.getDBID();
         // in-cluster distances
         DoubleMinMax iminmax = new DoubleMinMax();
-        for(DBID id2 : c1.getIDs()) {
+        for(DBIDIter iter2 = c1.getIDs().iter(); iter2.valid(); iter2.advance()) {
+          DBID id2 = iter2.getDBID();
           // skip the point itself.
-          if(id1 == id2) {
+          if(id1.sameDBID(id2)) {
             continue;
           }
           double d = distFunc.distance(id1, id2).doubleValue();
@@ -211,9 +211,10 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
           if(c2 == c1) {
             continue;
           }
-          for(DBID id2 : c2.getIDs()) {
+          for(DBIDIter iter2 = c2.getIDs().iter(); iter2.valid(); iter2.advance()) {
+            DBID id2 = iter2.getDBID();
             // skip the point itself (shouldn't happen though)
-            if(id1 == id2) {
+            if(id1.sameDBID(id2)) {
               continue;
             }
             double d = distFunc.distance(id1, id2).doubleValue();
@@ -249,21 +250,19 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     // count the number of samples we have in the data
     long inum = 0;
     long onum = 0;
-    for(Pair<Double, Pair<Long, Long>> ppair : histogram) {
+    for(DoubleObjPair<Pair<Long, Long>> ppair : histogram) {
       inum += ppair.getSecond().getFirst();
       onum += ppair.getSecond().getSecond();
     }
     long bnum = inum + onum;
-    // Note: when full sampling is added, this assertion won't hold anymore.
-    assert (bnum == relation.size() * (relation.size() - 1));
 
     Collection<DoubleVector> binstat = new ArrayList<DoubleVector>(numbin);
-    for(Pair<Double, Pair<Long, Long>> ppair : histogram) {
+    for(DoubleObjPair<Pair<Long, Long>> ppair : histogram) {
       final double icof = (inum == 0) ? 0 : ((double) ppair.getSecond().getFirst()) / inum / histogram.getBinsize();
       final double icaf = ((double) ppair.getSecond().getFirst()) / bnum / histogram.getBinsize();
       final double ocof = (onum == 0) ? 0 : ((double) ppair.getSecond().getSecond()) / onum / histogram.getBinsize();
       final double ocaf = ((double) ppair.getSecond().getSecond()) / bnum / histogram.getBinsize();
-      DoubleVector row = new DoubleVector(new double[] { ppair.getFirst(), icof, icaf, ocof, ocaf });
+      DoubleVector row = new DoubleVector(new double[] { ppair.first, icof, icaf, ocof, ocaf });
       binstat.add(row);
     }
     HistogramResult<DoubleVector> result = new HistogramResult<DoubleVector>("Distance Histogram", "distance-histogram", binstat);
@@ -284,58 +283,62 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     Random rnd = new Random();
     // estimate minimum and maximum.
     int k = (int) Math.max(25, Math.pow(database.size(), 0.2));
-    TreeSet<FCPair<Double, DBID>> minhotset = new TreeSet<FCPair<Double, DBID>>();
-    TreeSet<FCPair<Double, DBID>> maxhotset = new TreeSet<FCPair<Double, DBID>>(Collections.reverseOrder());
+    TreeSet<DoubleObjPair<DBID>> minhotset = new TreeSet<DoubleObjPair<DBID>>();
+    TreeSet<DoubleObjPair<DBID>> maxhotset = new TreeSet<DoubleObjPair<DBID>>(Collections.reverseOrder());
 
     int randomsize = (int) Math.max(25, Math.pow(database.size(), 0.2));
     double rprob = ((double) randomsize) / size;
     ArrayModifiableDBIDs randomset = DBIDUtil.newArray(randomsize);
 
-    Iterator<DBID> iter = database.iterDBIDs();
-    if(!iter.hasNext()) {
+    DBIDIter iter = database.iterDBIDs();
+    if(!iter.valid()) {
       throw new IllegalStateException(ExceptionMessages.DATABASE_EMPTY);
     }
-    DBID firstid = iter.next();
-    minhotset.add(new FCPair<Double, DBID>(Double.MAX_VALUE, firstid));
-    maxhotset.add(new FCPair<Double, DBID>(Double.MIN_VALUE, firstid));
-    while(iter.hasNext()) {
-      DBID id1 = iter.next();
+    DBID firstid = iter.getDBID();
+    iter.advance();
+    minhotset.add(new DoubleObjPair<DBID>(Double.MAX_VALUE, firstid));
+    maxhotset.add(new DoubleObjPair<DBID>(Double.MIN_VALUE, firstid));
+    while(iter.valid()) {
+      DBID id1 = iter.getDBID();
+      iter.advance();
       // generate candidates for min distance.
-      ArrayList<FCPair<Double, DBID>> np = new ArrayList<FCPair<Double, DBID>>(k * 2 + randomsize * 2);
-      for(FCPair<Double, DBID> pair : minhotset) {
+      ArrayList<DoubleObjPair<DBID>> np = new ArrayList<DoubleObjPair<DBID>>(k * 2 + randomsize * 2);
+      for(DoubleObjPair<DBID> pair : minhotset) {
         DBID id2 = pair.getSecond();
         // skip the object itself
         if(id1.compareTo(id2) == 0) {
           continue;
         }
         double d = distFunc.distance(id1, id2).doubleValue();
-        np.add(new FCPair<Double, DBID>(d, id1));
-        np.add(new FCPair<Double, DBID>(d, id2));
+        np.add(new DoubleObjPair<DBID>(d, id1));
+        np.add(new DoubleObjPair<DBID>(d, id2));
       }
-      for(DBID id2 : randomset) {
+      for(DBIDIter iter2 = randomset.iter(); iter2.valid(); iter2.advance()) {
+        DBID id2 = iter2.getDBID();
         double d = distFunc.distance(id1, id2).doubleValue();
-        np.add(new FCPair<Double, DBID>(d, id1));
-        np.add(new FCPair<Double, DBID>(d, id2));
+        np.add(new DoubleObjPair<DBID>(d, id1));
+        np.add(new DoubleObjPair<DBID>(d, id2));
       }
       minhotset.addAll(np);
       shrinkHeap(minhotset, k);
 
       // generate candidates for max distance.
-      ArrayList<FCPair<Double, DBID>> np2 = new ArrayList<FCPair<Double, DBID>>(k * 2 + randomsize * 2);
-      for(FCPair<Double, DBID> pair : minhotset) {
+      ArrayList<DoubleObjPair<DBID>> np2 = new ArrayList<DoubleObjPair<DBID>>(k * 2 + randomsize * 2);
+      for(DoubleObjPair<DBID> pair : minhotset) {
         DBID id2 = pair.getSecond();
         // skip the object itself
         if(id1.compareTo(id2) == 0) {
           continue;
         }
         double d = distFunc.distance(id1, id2).doubleValue();
-        np2.add(new FCPair<Double, DBID>(d, id1));
-        np2.add(new FCPair<Double, DBID>(d, id2));
+        np2.add(new DoubleObjPair<DBID>(d, id1));
+        np2.add(new DoubleObjPair<DBID>(d, id2));
       }
-      for(DBID id2 : randomset) {
+      for(DBIDIter iter2 = randomset.iter(); iter2.valid(); iter2.advance()) {
+        DBID id2 = iter2.getDBID();
         double d = distFunc.distance(id1, id2).doubleValue();
-        np.add(new FCPair<Double, DBID>(d, id1));
-        np.add(new FCPair<Double, DBID>(d, id2));
+        np.add(new DoubleObjPair<DBID>(d, id1));
+        np.add(new DoubleObjPair<DBID>(d, id2));
       }
       maxhotset.addAll(np2);
       shrinkHeap(maxhotset, k);
@@ -348,14 +351,16 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
         randomset.set((int) Math.floor(rnd.nextDouble() * randomsize), id1);
       }
     }
-    return new DoubleMinMax(minhotset.first().getFirst(), maxhotset.first().getFirst());
+    return new DoubleMinMax(minhotset.first().first, maxhotset.first().first);
   }
 
   private DoubleMinMax exactMinMax(Relation<O> database, DistanceQuery<O, D> distFunc) {
     DoubleMinMax minmax = new DoubleMinMax();
     // find exact minimum and maximum first.
-    for(DBID id1 : database.iterDBIDs()) {
-      for(DBID id2 : database.iterDBIDs()) {
+    for(DBIDIter iditer = database.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      DBID id1  = iditer.getDBID();
+      for(DBIDIter iditer2 = database.iterDBIDs(); iditer2.valid(); iditer2.advance()) {
+        DBID id2  = iditer2.getDBID();
         // skip the point itself.
         if(id1.compareTo(id2) == 0) {
           continue;
@@ -367,12 +372,12 @@ public class DistanceStatisticsWithClasses<O, D extends NumberDistance<D, ?>> ex
     return minmax;
   }
 
-  private void shrinkHeap(TreeSet<FCPair<Double, DBID>> hotset, int k) {
+  private void shrinkHeap(TreeSet<DoubleObjPair<DBID>> hotset, int k) {
     // drop duplicates
     ModifiableDBIDs seenids = DBIDUtil.newHashSet(2 * k);
     int cnt = 0;
-    for(Iterator<FCPair<Double, DBID>> i = hotset.iterator(); i.hasNext();) {
-      FCPair<Double, DBID> p = i.next();
+    for(Iterator<DoubleObjPair<DBID>> i = hotset.iterator(); i.hasNext();) {
+      DoubleObjPair<DBID> p = i.next();
       if(cnt > k || seenids.contains(p.getSecond())) {
         i.remove();
       }
