@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.application.visualization;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -29,8 +29,6 @@ import java.awt.Component;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.HashMap;
-import java.util.List;
-import java.util.ListIterator;
 
 import javax.swing.DefaultListCellRenderer;
 import javax.swing.DefaultListModel;
@@ -59,15 +57,18 @@ import de.lmu.ifi.dbs.elki.data.VectorUtil;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.StaticArrayDatabase;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
+import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
+import de.lmu.ifi.dbs.elki.math.scales.LinearScale;
 import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
@@ -79,7 +80,6 @@ import de.lmu.ifi.dbs.elki.visualization.batikutil.LazyCanvasResizer;
 import de.lmu.ifi.dbs.elki.visualization.batikutil.NodeReplacer;
 import de.lmu.ifi.dbs.elki.visualization.css.CSSClassManager.CSSNamingConflict;
 import de.lmu.ifi.dbs.elki.visualization.savedialog.SVGSaveDialog;
-import de.lmu.ifi.dbs.elki.visualization.scales.LinearScale;
 import de.lmu.ifi.dbs.elki.visualization.style.PropertiesBasedStyleLibrary;
 import de.lmu.ifi.dbs.elki.visualization.style.StyleLibrary;
 import de.lmu.ifi.dbs.elki.visualization.svg.SVGPath;
@@ -376,7 +376,8 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
 
       double min = Double.MAX_VALUE;
       double max = Double.MIN_VALUE;
-      for(DBID objID : data.iterDBIDs()) {
+      for(DBIDIter iditer = data.iterDBIDs(); iditer.valid(); iditer.advance()) {
+        DBID objID  = iditer.getDBID();
         O vec = data.get(objID);
         DoubleMinMax mm = VectorUtil.getRangeDouble(vec);
         min = Math.min(min, mm.getMin());
@@ -393,7 +394,7 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
 
       try {
         StyleLibrary style = new PropertiesBasedStyleLibrary();
-        SVGSimpleLinearAxis.drawAxis(plot, viewport, this.s, 0.0, StyleLibrary.SCALE, 0.0, 0.0, true, false, style);
+        SVGSimpleLinearAxis.drawAxis(plot, viewport, this.s, 0.0, StyleLibrary.SCALE, 0.0, 0.0, SVGSimpleLinearAxis.LabelStyle.LEFTHAND, style);
       }
       catch(CSSNamingConflict e) {
         LoggingUtil.exception(e);
@@ -409,7 +410,8 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
       svgCanvas.setPlot(plot);
 
       DefaultListModel m = new DefaultListModel();
-      for(DBID dbid : data.iterDBIDs()) {
+      for(DBIDIter iditer = data.iterDBIDs(); iditer.valid(); iditer.advance()) {
+        DBID dbid  = iditer.getDBID();
         m.addElement(dbid);
       }
       seriesList.setModel(m);
@@ -436,21 +438,21 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
       for(Object o : sel) {
         DBID idx = (DBID) o;
 
-        List<DistanceResultPair<D>> knn = knnQuery.getKNNForDBID(idx, k);
+        KNNResult<D> knn = knnQuery.getKNNForDBID(idx, k);
 
-        double maxdist = knn.get(knn.size() - 1).getDistance().doubleValue();
+        double maxdist = knn.getKNNDistance().doubleValue();
         // avoid division by zero.
         if(maxdist == 0) {
           maxdist = 1;
         }
 
-        for(ListIterator<DistanceResultPair<D>> iter = knn.listIterator(knn.size()); iter.hasPrevious();) {
-          DistanceResultPair<D> pair = iter.previous();
+        for (int i = knn.size() - 1; i >= 0; i--) {
+          DistanceResultPair<D> pair = knn.get(i);
           Element line = plotSeries(pair.getDBID(), MAXRESOLUTION);
           double dist = pair.getDistance().doubleValue() / maxdist;
           Color color = getColor(dist);
           String colstr = "#" + Integer.toHexString(color.getRGB()).substring(2);
-          String width = (pair.getDBID() == idx) ? "0.5%" : "0.2%";
+          String width = (pair.getDBID().sameDBID(idx)) ? "0.5%" : "0.2%";
           SVGUtil.setStyle(line, "stroke: " + colstr + "; stroke-width: " + width + "; fill: none");
           newe.appendChild(line);
           // put into cache
