@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -25,7 +25,6 @@ package de.lmu.ifi.dbs.elki.database;
 
 import java.util.BitSet;
 import java.util.Collection;
-import java.util.Collections;
 
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
@@ -33,7 +32,6 @@ import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
-import de.lmu.ifi.dbs.elki.database.ids.StaticDBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.DBIDView;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
@@ -65,7 +63,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
  * @apiviz.uses DatabaseConnection
  */
 @Description("Database using an in-memory hashtable and at least providing linear scans.")
-public class StaticArrayDatabase extends AbstractDatabase implements Database, Parameterizable {
+public class StaticArrayDatabase extends AbstractDatabase implements Parameterizable {
   /**
    * Our logger
    */
@@ -185,21 +183,14 @@ public class StaticArrayDatabase extends AbstractDatabase implements Database, P
 
   @Override
   public void addIndex(Index index) {
+    if(logger.isDebuggingFiner()) {
+      logger.debugFine("Adding index: " + index);
+    }
     this.indexes.add(index);
     // TODO: actually add index to the representation used?
     this.addChildResult(index);
   }
 
-  @Override
-  public Collection<Index> getIndexes() {
-    return Collections.unmodifiableList(this.indexes);
-  }
-
-  @Override
-  public void removeIndex(Index index) {
-    this.indexes.remove(index);
-    this.getHierarchy().remove(this, index);
-  }
 
   /**
    * Find an DBID column.
@@ -263,18 +254,6 @@ public class StaticArrayDatabase extends AbstractDatabase implements Database, P
     return relation;
   }
 
-  @Deprecated
-  @Override
-  public final int size() {
-    return ids.size();
-  }
-
-  @Deprecated
-  @Override
-  public StaticDBIDs getDBIDs() {
-    return DBIDUtil.makeUnmodifiable(ids);
-  }
-
   @Override
   protected Logging getLogger() {
     return logger;
