@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,20 +23,45 @@ package de.lmu.ifi.dbs.elki.database.ids;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.List;
-
 /**
  * Interface for array based DBIDs.
  * 
  * @author Erich Schubert
  */
-public interface ArrayDBIDs extends DBIDs, List<DBID> {
+public interface ArrayDBIDs extends DBIDs {
   /**
    * Get the i'th entry (starting at 0)
    * 
    * @param i Index
    * @return DBID of i'th entry.
    */
-  // In List<DBID> which confuses the java compiler
-  /* public DBID get(int i); */
+  public DBID get(int i);
+
+  /**
+   * Iterable
+   * 
+   * @return Iterator
+   */
+  @Override
+  public DBIDIter iter();
+
+  /**
+   * Size of the DBID "collection".
+   * 
+   * @return size
+   */
+  @Override
+  public int size();
+
+  /**
+   * Search for the position of the given key, assuming that the data set is
+   * sorted.
+   * 
+   * For keys not found, <code>-(1+insertion position)</code> is returned, as
+   * for Java {@link java.util.Collections#binarySearch}
+   * 
+   * @param key Key to search for
+   * @return Offset of key
+   */
+  public int binarySearch(DBIDRef key);
 }
