@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,7 +23,6 @@ package de.lmu.ifi.dbs.elki.database.ids;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 /**
  * Static DBID range.
  * 
@@ -39,5 +38,5 @@ public interface DBIDRange extends ArrayStaticDBIDs {
    * @param dbid ID to compute index for
    * @return index
    */
-  public int getOffset(DBID dbid);
+  public int getOffset(DBIDRef dbid);
 }
\ No newline at end of file
