@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,14 +23,12 @@ package de.lmu.ifi.dbs.elki.database.ids;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.Set;
-
 /**
  * Set-oriented implementation of a modifiable DBID collection.
  * 
  * @author Erich Schubert
  */
-public interface HashSetModifiableDBIDs extends Set<DBID>, HashSetDBIDs, ModifiableDBIDs {
+public interface HashSetModifiableDBIDs extends HashSetDBIDs, ModifiableDBIDs {
   /**
    * Retain all elements that also are in the second set.
    * 
