@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,14 +23,16 @@ package de.lmu.ifi.dbs.elki.database.ids;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.Collection;
-
 /**
  * Interface for a generic modifiable DBID collection.
  * 
+ * Note: we had this use the Java Collections API for a long time, however this
+ * prevented certain optimizations. So it now only mimics the Collections API
+ * <em>deliberately</em>.
+ * 
  * @author Erich Schubert
  */
-public interface ModifiableDBIDs extends DBIDs, Collection<DBID> {
+public interface ModifiableDBIDs extends DBIDs {
   /**
    * Add DBIDs to collection.
    * 
@@ -38,7 +40,7 @@ public interface ModifiableDBIDs extends DBIDs, Collection<DBID> {
    * @return {@code true} when modified
    */
   boolean addDBIDs(DBIDs ids);
-  
+
   /**
    * Remove DBIDs from collection.
    * 
@@ -46,4 +48,42 @@ public interface ModifiableDBIDs extends DBIDs, Collection<DBID> {
    * @return {@code true} when modified
    */
   boolean removeDBIDs(DBIDs ids);
-}
+
+  /**
+   * Add a single DBID to the collection.
+   * 
+   * @param id ID to add
+   */
+  boolean add(DBIDRef id);
+
+  /**
+   * Remove a single DBID from the collection.
+   * 
+   * @param id ID to remove
+   */
+  boolean remove(DBIDRef id);
+
+  /**
+   * Clear this collection.
+   */
+  void clear();
+
+  /**
+   * Get a <em>modifiable</em> DBID iterator (a more efficient API).
+   * 
+   * usage example:
+   * 
+   * <pre>
+   * {@code
+   * for(DBIDMIter iter = ids.iter(); iter.valid(); iter.advance()) {
+   *   DBID id = iter.getDBID();
+   *   iter.remove();
+   * }
+   * }
+   * </pre>
+   * 
+   * @return modifiable iterator
+   */
+  @Override
+  DBIDMIter iter();
+}
\ No newline at end of file
