@@ -1,9 +1,10 @@
 package de.lmu.ifi.dbs.elki.datasource;
+
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -28,7 +29,8 @@ import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
-import java.util.LinkedList;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.Random;
 import java.util.regex.Pattern;
 
@@ -47,13 +49,13 @@ import de.lmu.ifi.dbs.elki.application.GeneratorXMLSpec;
 import de.lmu.ifi.dbs.elki.data.synthetic.bymodel.GeneratorMain;
 import de.lmu.ifi.dbs.elki.data.synthetic.bymodel.GeneratorSingleCluster;
 import de.lmu.ifi.dbs.elki.data.synthetic.bymodel.GeneratorStatic;
-import de.lmu.ifi.dbs.elki.data.synthetic.bymodel.distribution.Distribution;
-import de.lmu.ifi.dbs.elki.data.synthetic.bymodel.distribution.GammaDistribution;
-import de.lmu.ifi.dbs.elki.data.synthetic.bymodel.distribution.NormalDistribution;
-import de.lmu.ifi.dbs.elki.data.synthetic.bymodel.distribution.UniformDistribution;
 import de.lmu.ifi.dbs.elki.datasource.bundle.MultipleObjectsBundle;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
+import de.lmu.ifi.dbs.elki.math.statistics.distribution.DistributionWithRandom;
+import de.lmu.ifi.dbs.elki.math.statistics.distribution.GammaDistribution;
+import de.lmu.ifi.dbs.elki.math.statistics.distribution.NormalDistribution;
+import de.lmu.ifi.dbs.elki.math.statistics.distribution.UniformDistribution;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.UnableToComplyException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
@@ -64,6 +66,16 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.FileParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.xml.XMLNodeIterator;
 
+/**
+ * Data source from an XML specification.
+ * 
+ * This data source will generate random (or pseudo-random, fixed seeds are
+ * supported) data sets that satisfy a given specification file.
+ * 
+ * @author Erich Schubert
+ * 
+ * @apiviz.composedOf GeneratorMain
+ */
 public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
   /**
    * Logger
@@ -105,11 +117,6 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
    */
   double sizescale = 1.0;
 
-  /**
-   * The actual generator class
-   */
-  public GeneratorMain gen = new GeneratorMain();
-
   /**
    * Random generator used for initializing cluster generators.
    */
@@ -118,7 +125,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
   /**
    * Set testAgainstModel flag
    */
-  private boolean testAgainstModel = true;
+  private Boolean testAgainstModel;
 
   /**
    * Constructor.
@@ -142,8 +149,9 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     if(logger.isVerbose()) {
       logger.verbose("Loading specification ...");
     }
+    GeneratorMain gen;
     try {
-      loadXMLSpecification();
+      gen = loadXMLSpecification();
     }
     catch(UnableToComplyException e) {
       throw new AbortException("Cannot load XML specification", e);
@@ -151,10 +159,11 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     if(logger.isVerbose()) {
       logger.verbose("Generating clusters ...");
     }
-    gen.setTestAgainstModel(testAgainstModel);
+    if(testAgainstModel != null) {
+      gen.setTestAgainstModel(testAgainstModel);
+    }
     try {
-      gen.generate();
-      return gen.getBundle();
+      return gen.generate();
     }
     catch(UnableToComplyException e) {
       throw new AbortException("Data generation failed. ", e);
@@ -165,8 +174,10 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
    * Load the XML configuration file.
    * 
    * @throws UnableToComplyException
+   * 
+   * @return Generator
    */
-  private void loadXMLSpecification() throws UnableToComplyException {
+  private GeneratorMain loadXMLSpecification() throws UnableToComplyException {
     try {
       InputStream in = new FileInputStream(specfile);
       DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
@@ -188,7 +199,9 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
       Document doc = dbf.newDocumentBuilder().parse(in);
       Node root = doc.getDocumentElement();
       if(root.getNodeName() == "dataset") {
-        processElementDataset(root);
+        GeneratorMain gen = new GeneratorMain();
+        processElementDataset(gen, root);
+        return gen;
       }
       else {
         throw new UnableToComplyException("Experiment specification has incorrect document element: " + root.getNodeName());
@@ -211,10 +224,11 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
   /**
    * Process a 'dataset' Element in the XML stream.
    * 
+   * @param gen Generator
    * @param cur Current document nod
    * @throws UnableToComplyException
    */
-  private void processElementDataset(Node cur) throws UnableToComplyException {
+  private void processElementDataset(GeneratorMain gen, Node cur) throws UnableToComplyException {
     // *** get parameters
     String seedstr = ((Element) cur).getAttribute("random-seed");
     if(seedstr != null && seedstr != "") {
@@ -225,12 +239,14 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
       testAgainstModel = (Integer.valueOf(testmod) != 0);
     }
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeName() == "cluster") {
-        processElementCluster(child);
+        processElementCluster(gen, child);
       }
       else if(child.getNodeName() == "static") {
-        processElementStatic(child);
+        processElementStatic(gen, child);
       }
       else if(child.getNodeType() == Node.ELEMENT_NODE) {
         logger.warning("Unknown element in XML specification file: " + child.getNodeName());
@@ -241,10 +257,11 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
   /**
    * Process a 'cluster' Element in the XML stream.
    * 
+   * @param gen Generator
    * @param cur Current document nod
    * @throws UnableToComplyException
    */
-  private void processElementCluster(Node cur) throws UnableToComplyException {
+  private void processElementCluster(GeneratorMain gen, Node cur) throws UnableToComplyException {
     int size = -1;
     double overweight = 1.0;
 
@@ -257,7 +274,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     String dcostr = ((Element) cur).getAttribute("density-correction");
     if(dcostr != null && dcostr != "") {
-      overweight = Double.valueOf(dcostr);
+      overweight = Double.parseDouble(dcostr);
     }
 
     if(size < 0) {
@@ -272,7 +289,9 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     GeneratorSingleCluster cluster = new GeneratorSingleCluster(name, size, overweight, newRand);
 
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeName() == "uniform") {
         processElementUniform(cluster, child);
       }
@@ -312,20 +331,22 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     String minstr = ((Element) cur).getAttribute("min");
     if(minstr != null && minstr != "") {
-      min = Double.valueOf(minstr);
+      min = Double.parseDouble(minstr);
     }
     String maxstr = ((Element) cur).getAttribute("max");
     if(maxstr != null && maxstr != "") {
-      max = Double.valueOf(maxstr);
+      max = Double.parseDouble(maxstr);
     }
 
     // *** new uniform generator
     Random random = cluster.getNewRandomGenerator();
-    Distribution generator = new UniformDistribution(min, max, random);
+    DistributionWithRandom generator = new UniformDistribution(min, max, random);
     cluster.addGenerator(generator);
 
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeType() == Node.ELEMENT_NODE) {
         logger.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
@@ -344,20 +365,22 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     double stddev = 1.0;
     String meanstr = ((Element) cur).getAttribute("mean");
     if(meanstr != null && meanstr != "") {
-      mean = Double.valueOf(meanstr);
+      mean = Double.parseDouble(meanstr);
     }
     String stddevstr = ((Element) cur).getAttribute("stddev");
     if(stddevstr != null && stddevstr != "") {
-      stddev = Double.valueOf(stddevstr);
+      stddev = Double.parseDouble(stddevstr);
     }
 
     // *** New normal distribution generator
     Random random = cluster.getNewRandomGenerator();
-    Distribution generator = new NormalDistribution(mean, stddev, random);
+    DistributionWithRandom generator = new NormalDistribution(mean, stddev, random);
     cluster.addGenerator(generator);
 
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeType() == Node.ELEMENT_NODE) {
         logger.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
@@ -376,20 +399,22 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     double theta = 1.0;
     String kstr = ((Element) cur).getAttribute("k");
     if(kstr != null && kstr != "") {
-      k = Double.valueOf(kstr);
+      k = Double.parseDouble(kstr);
     }
     String thetastr = ((Element) cur).getAttribute("theta");
     if(thetastr != null && thetastr != "") {
-      theta = Double.valueOf(thetastr);
+      theta = Double.parseDouble(thetastr);
     }
 
     // *** New normal distribution generator
     Random random = cluster.getNewRandomGenerator();
-    Distribution generator = new GammaDistribution(k, theta, random);
+    DistributionWithRandom generator = new GammaDistribution(k, theta, random);
     cluster.addGenerator(generator);
 
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeType() == Node.ELEMENT_NODE) {
         logger.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
@@ -418,7 +443,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     }
     String anstr = ((Element) cur).getAttribute("angle");
     if(anstr != null && anstr != "") {
-      angle = Double.valueOf(anstr);
+      angle = Double.parseDouble(anstr);
     }
     if(axis1 <= 0 || axis1 > cluster.getDim()) {
       throw new UnableToComplyException("Invalid axis1 number given in specification file.");
@@ -434,7 +459,9 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     cluster.addRotation(axis1 - 1, axis2 - 1, Math.toRadians(angle));
 
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeType() == Node.ELEMENT_NODE) {
         logger.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
@@ -462,7 +489,9 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     cluster.addTranslation(offset);
 
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeType() == Node.ELEMENT_NODE) {
         logger.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
@@ -496,7 +525,9 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     cluster.setClipping(cmin, cmax);
 
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeType() == Node.ELEMENT_NODE) {
         logger.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
@@ -506,18 +537,21 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
   /**
    * Process a 'static' cluster Element in the XML stream.
    * 
+   * @param gen Generator
    * @param cur Current document nod
    * @throws UnableToComplyException
    */
-  private void processElementStatic(Node cur) throws UnableToComplyException {
+  private void processElementStatic(GeneratorMain gen, Node cur) throws UnableToComplyException {
     String name = ((Element) cur).getAttribute("name");
     if(name == null) {
       throw new UnableToComplyException("No cluster name given in specification file.");
     }
 
-    LinkedList<Vector> points = new LinkedList<Vector>();
+    ArrayList<Vector> points = new ArrayList<Vector>();
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeName() == "point") {
         processElementPoint(points, child);
       }
@@ -541,7 +575,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
    * @param cur Current document nod
    * @throws UnableToComplyException
    */
-  private void processElementPoint(LinkedList<Vector> points, Node cur) throws UnableToComplyException {
+  private void processElementPoint(List<Vector> points, Node cur) throws UnableToComplyException {
     Vector point = null;
     String vstr = ((Element) cur).getAttribute("vector");
     if(vstr != null && vstr != "") {
@@ -555,7 +589,9 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     points.add(point);
 
     // TODO: check for unknown attributes.
-    for(Node child : new XMLNodeIterator(cur.getFirstChild())) {
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while(iter.hasNext()) {
+      Node child = iter.next();
       if(child.getNodeType() == Node.ELEMENT_NODE) {
         logger.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
@@ -576,7 +612,7 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     double[] d = new double[entries.length];
     for(int i = 0; i < entries.length; i++) {
       try {
-        d[i] = Double.valueOf(entries[i]);
+        d[i] = Double.parseDouble(entries[i]);
       }
       catch(NumberFormatException e) {
         throw new UnableToComplyException("Could not parse vector.");
