@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -76,10 +76,9 @@ public class RandomDoubleVectorDatabaseConnection extends AbstractDatabaseConnec
 
   private static final Logging logger = Logging.getLogger(RandomDoubleVectorDatabaseConnection.class);
 
-  
   @Override
   public MultipleObjectsBundle loadData() {
-    VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation(DoubleVector.class, dim, new DoubleVector(new double[dim]));
+    VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.class, dim, DoubleVector.STATIC);
     List<DoubleVector> vectors = new ArrayList<DoubleVector>(size);
 
     // Setup random generator
@@ -178,4 +177,4 @@ public class RandomDoubleVectorDatabaseConnection extends AbstractDatabaseConnec
       return new RandomDoubleVectorDatabaseConnection(dim, size, seed, filters);
     }
   }
-}
+}
\ No newline at end of file
