@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.datasource.parser;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,17 +23,16 @@ package de.lmu.ifi.dbs.elki.datasource.parser;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import gnu.trove.iterator.TIntObjectIterator;
+import gnu.trove.map.hash.TIntFloatHashMap;
+import gnu.trove.map.hash.TIntObjectHashMap;
+
 import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.StreamTokenizer;
 import java.util.ArrayList;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.Map;
-import java.util.Map.Entry;
-import java.util.TreeMap;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -61,7 +60,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.PatternParameter;
  * This parser is quite hackish, and contains lots of not yet configurable
  * magic.
  * 
- * TODO: Sparse vectors are not yet supported.
+ * TODO: Sparse vectors are not yet fully supported.
  * 
  * @author Erich Schubert
  */
@@ -94,7 +93,7 @@ public class ArffParser implements Parser {
   /**
    * Pattern to auto-convert columns to external ids.
    */
-  public static final String DEFAULT_ARFF_MAGIC_EID = "(ID|External-?ID)";
+  public static final String DEFAULT_ARFF_MAGIC_EID = "(External-?ID)";
 
   /**
    * Pattern to auto-convert columns to class labels.
@@ -206,7 +205,7 @@ public class ArffParser implements Parser {
 
   private Object[] loadSparseInstance(StreamTokenizer tokenizer, int[] targ, int[] dimsize, TypeInformation[] elkitypes, int metaLength) throws IOException {
     // logger.warning("Sparse instance.");
-    Map<Integer, Object> map = new TreeMap<Integer, Object>();
+    TIntObjectHashMap<Object> map = new TIntObjectHashMap<Object>();
     while(true) {
       nextToken(tokenizer);
       assert (tokenizer.ttype != StreamTokenizer.TT_EOF && tokenizer.ttype != StreamTokenizer.TT_EOL);
@@ -248,32 +247,34 @@ public class ArffParser implements Parser {
       }
       assert (s >= 0);
       if(elkitypes[out] == TypeUtil.NUMBER_VECTOR_FIELD) {
-        Map<Integer, Float> f = new HashMap<Integer, Float>(dimsize[out]);
-        for(Entry<Integer, Object> key : map.entrySet()) {
-          int i = key.getKey();
+        TIntFloatHashMap f = new TIntFloatHashMap(dimsize[out]);
+        for(TIntObjectIterator<Object> iter = map.iterator(); iter.hasNext();) {
+          iter.advance();
+          int i = iter.key();
           if(i < s) {
             continue;
           }
           if(i >= s + dimsize[out]) {
             break;
           }
-          double v = (Double) key.getValue();
+          double v = (Double) iter.value();
           f.put(i - s + 1, (float) v);
         }
         data[out] = new SparseFloatVector(f, dimsize[out]);
       }
       else if(elkitypes[out] == TypeUtil.LABELLIST) {
         // Build a label list out of successive labels
-        LabelList ll = new LabelList();
-        for(Entry<Integer, Object> key : map.entrySet()) {
-          int i = key.getKey();
+        LabelList ll = new LabelList(1);
+        for(TIntObjectIterator<Object> iter = map.iterator(); iter.hasNext();) {
+          iter.advance();
+          int i = iter.key();
           if(i < s) {
             continue;
           }
           if(i >= s + dimsize[out]) {
             break;
           }
-          String v = (String) key.getValue();
+          String v = (String) iter.value();
           if(ll.size() < i - s) {
             logger.warning("Sparse consecutive labels are currently not correctly supported.");
           }
@@ -309,24 +310,33 @@ public class ArffParser implements Parser {
   }
 
   private Object[] loadDenseInstance(StreamTokenizer tokenizer, int[] dimsize, TypeInformation[] etyp, int outdim) throws IOException {
-    // logger.warning("Regular instance.");
     Object[] data = new Object[outdim];
     for(int out = 0; out < outdim; out++) {
       if(etyp[out] == TypeUtil.NUMBER_VECTOR_FIELD) {
         // For multi-column vectors, read successive columns
         double[] cur = new double[dimsize[out]];
         for(int k = 0; k < dimsize[out]; k++) {
-          if(tokenizer.ttype != StreamTokenizer.TT_NUMBER) {
+          if(tokenizer.ttype == '?') {
+            tokenizer.nval = Double.NaN;
+          }
+          else if(tokenizer.ttype == StreamTokenizer.TT_WORD) {
+            try {
+              cur[k] = Double.parseDouble(tokenizer.sval);
+            }
+            catch(NumberFormatException e) {
+              throw new AbortException("Expected number value, got: " + tokenizer.sval);
+            }
+          }
+          else {
             throw new AbortException("Expected word token, got: " + tokenizer.toString());
           }
-          cur[k] = tokenizer.nval;
           nextToken(tokenizer);
         }
         data[out] = new DoubleVector(cur);
       }
       else if(etyp[out] == TypeUtil.LABELLIST) {
         // Build a label list out of successive labels
-        LabelList ll = new LabelList();
+        LabelList ll = new LabelList(dimsize[out]);
         for(int k = 0; k < dimsize[out]; k++) {
           if(tokenizer.ttype != StreamTokenizer.TT_WORD) {
             throw new AbortException("Expected word token, got: " + tokenizer.toString());
@@ -369,7 +379,11 @@ public class ArffParser implements Parser {
     // Setup tokenizer
     StreamTokenizer tokenizer = new StreamTokenizer(br);
     {
+      tokenizer.resetSyntax();
       tokenizer.whitespaceChars(0, ' ');
+      tokenizer.ordinaryChars('0', '9');
+      tokenizer.ordinaryChar('-');
+      tokenizer.ordinaryChar('.');
       tokenizer.wordChars(' ' + 1, '\u00FF');
       tokenizer.whitespaceChars(',', ',');
       tokenizer.commentChar('%');
@@ -407,12 +421,11 @@ public class ArffParser implements Parser {
           labels[i] = names.get(out + i);
         }
         if(!sparse) {
-          VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.class, dimsize[out], labels, new DoubleVector(new double[dimsize[out]]));
+          VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.class, DoubleVector.STATIC, dimsize[out], labels, new DoubleVector(new double[dimsize[out]]));
           bundle.appendColumn(type, new ArrayList<DoubleVector>());
         }
         else {
-          Map<Integer, Float> empty = Collections.emptyMap();
-          VectorFieldTypeInformation<SparseFloatVector> type = new VectorFieldTypeInformation<SparseFloatVector>(SparseFloatVector.class, dimsize[out], labels, new SparseFloatVector(empty, dimsize[out]));
+          VectorFieldTypeInformation<SparseFloatVector> type = new VectorFieldTypeInformation<SparseFloatVector>(SparseFloatVector.class, dimsize[out], labels, new SparseFloatVector(SparseFloatVector.EMPTYMAP, dimsize[out]));
           bundle.appendColumn(type, new ArrayList<SparseFloatVector>());
         }
       }
@@ -585,20 +598,22 @@ public class ArffParser implements Parser {
     else if((tokenizer.ttype == StreamTokenizer.TT_WORD) && (tokenizer.sval.equals("?"))) {
       tokenizer.ttype = '?';
     }
-    if(tokenizer.ttype == StreamTokenizer.TT_NUMBER) {
-      logger.debug("token: " + tokenizer.nval);
-    }
-    else if(tokenizer.ttype == StreamTokenizer.TT_WORD) {
-      logger.debug("token: " + tokenizer.sval);
-    }
-    else if(tokenizer.ttype == StreamTokenizer.TT_EOF) {
-      logger.debug("token: EOF");
-    }
-    else if(tokenizer.ttype == StreamTokenizer.TT_EOL) {
-      logger.debug("token: EOL");
-    }
-    else {
-      logger.debug("token type: " + tokenizer.ttype);
+    if(logger.isDebugging()) {
+      if(tokenizer.ttype == StreamTokenizer.TT_NUMBER) {
+        logger.debug("token: " + tokenizer.nval);
+      }
+      else if(tokenizer.ttype == StreamTokenizer.TT_WORD) {
+        logger.debug("token: " + tokenizer.sval);
+      }
+      else if(tokenizer.ttype == StreamTokenizer.TT_EOF) {
+        logger.debug("token: EOF");
+      }
+      else if(tokenizer.ttype == StreamTokenizer.TT_EOL) {
+        logger.debug("token: EOL");
+      }
+      else {
+        logger.debug("token type: " + tokenizer.ttype);
+      }
     }
   }
 
@@ -606,7 +621,7 @@ public class ArffParser implements Parser {
    * Parameterization class.
    * 
    * @author Erich Schubert
-   *
+   * 
    * @apiviz.exclude
    */
   public static class Parameterizer extends AbstractParameterizer {
