@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.evaluation.roc;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,21 +23,21 @@ package de.lmu.ifi.dbs.elki.evaluation.roc;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.ArrayList;
 import java.util.Iterator;
-import java.util.List;
 import java.util.Set;
 
 import de.lmu.ifi.dbs.elki.data.Cluster;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDPair;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.SetDBIDs;
 import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
+import de.lmu.ifi.dbs.elki.math.geometry.XYCurve;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
-import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleDoublePair;
 import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
 import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 import de.lmu.ifi.dbs.elki.utilities.pairs.PairInterface;
@@ -80,23 +80,16 @@ public class ROC {
    *        'same positions'.
    * @return area under curve
    */
-  public static <C extends Comparable<? super C>, T> List<DoubleDoublePair> materializeROC(int size, Set<? super T> ids, Iterator<? extends PairInterface<C, T>> nei) {
-    int postot = ids.size();
-    int negtot = size - postot;
-    int poscnt = 0;
-    int negcnt = 0;
-    ArrayList<DoubleDoublePair> res = new ArrayList<DoubleDoublePair>(postot + 2);
+  public static <C extends Comparable<? super C>, T> XYCurve materializeROC(int size, Set<? super T> ids, Iterator<? extends PairInterface<C, T>> nei) {
+    final int postot = ids.size(), negtot = size - postot;
+    int poscnt = 0, negcnt = 0;
+    XYCurve curve = new XYCurve("True Negative Rate", "True Positive Rate", postot + 2);
 
     // start in bottom left
-    res.add(new DoubleDoublePair(0.0, 0.0));
+    curve.add(0.0, 0.0);
 
-    PairInterface<C, T> prev = null;
+    C prevval = null;
     while(nei.hasNext()) {
-      // Previous positive rate - y axis
-      double curpos = ((double) poscnt) / postot;
-      // Previous negative rate - x axis
-      double curneg = ((double) negcnt) / negtot;
-
       // Analyze next point
       PairInterface<C, T> cur = nei.next();
       // positive or negative match?
@@ -107,46 +100,65 @@ public class ROC {
         negcnt += 1;
       }
       // defer calculation for ties
-      if((prev != null) && (prev.getFirst().compareTo(cur.getFirst()) == 0)) {
+      if((prevval != null) && (prevval.compareTo(cur.getFirst()) == 0)) {
         continue;
       }
-      // simplify curve when possible:
-      if(res.size() >= 2) {
-        DoubleDoublePair last1 = res.get(res.size() - 2);
-        DoubleDoublePair last2 = res.get(res.size() - 1);
-        final double ldx = last2.first - last1.first;
-        final double cdx = curneg - last2.first;
-        final double ldy = last2.second - last1.second;
-        final double cdy = curpos - last2.second;
-        // vertical simplification
-        if((ldx == 0) && (cdx == 0)) {
-          res.remove(res.size() - 1);
-        }
-        // horizontal simplification
-        else if((ldy == 0) && (cdy == 0)) {
-          res.remove(res.size() - 1);
-        }
-        // diagonal simplification
-        else if(ldy > 0 && cdy > 0) {
-          if(Math.abs((ldx / ldy) - (cdx / cdy)) < 1E-10) {
-            res.remove(res.size() - 1);
-          }
-        }
-      }
-      // Add a new point (for the previous entry!)
-      res.add(new DoubleDoublePair(curneg, curpos));
-      prev = cur;
+      // Add a new point.
+      curve.addAndSimplify(negcnt / (double) negtot, poscnt / (double) postot);
+      prevval = cur.getFirst();
     }
-    // ensure we end up in the top right corner.
-    // Since we didn't add a point for the last entry yet, this likely is
-    // needed.
-    {
-      DoubleDoublePair last = res.get(res.size() - 1);
-      if(last.first < 1.0 || last.second < 1.0) {
-        res.add(new DoubleDoublePair(1.0, 1.0));
+    // Ensure we end up in the top right corner.
+    // Simplification will skip this if we already were.
+    curve.addAndSimplify(1.0, 1.0);
+    return curve;
+  }
+
+  /**
+   * Compute a ROC curve given a set of positive IDs and a sorted list of
+   * (comparable, ID)s, where the comparable object is used to decided when two
+   * objects are interchangeable.
+   * 
+   * @param <C> Reference type
+   * @param size Database size
+   * @param ids Collection of positive IDs, should support efficient contains()
+   * @param nei List of neighbors along with some comparable object to detect
+   *        'same positions'.
+   * @return area under curve
+   */
+  public static <C extends Comparable<? super C>> XYCurve materializeROC(int size, SetDBIDs ids, Iterator<? extends PairInterface<C, DBID>> nei) {
+    final int postot = ids.size(), negtot = size - postot;
+    int poscnt = 0, negcnt = 0;
+    XYCurve curve = new XYCurve("True Negative Rate", "True Positive Rate", postot + 2);
+
+    // start in bottom left
+    curve.add(0.0, 0.0);
+
+    C prevval = null;
+    while(nei.hasNext()) {
+      // Rates at *previous* data point. Because of tie handling strategy!
+      final double trueneg = negcnt / (double) negtot;
+      final double truepos = poscnt / (double) postot;
+      // Analyze next point
+      PairInterface<C, DBID> cur = nei.next();
+      // positive or negative match?
+      if(ids.contains(cur.getSecond())) {
+        poscnt += 1;
       }
+      else {
+        negcnt += 1;
+      }
+      // defer calculation for ties
+      if((prevval != null) && (prevval.compareTo(cur.getFirst()) == 0)) {
+        continue;
+      }
+      // Add point for *previous* result (since we are no longer tied with it)
+      curve.addAndSimplify(trueneg, truepos);
+      prevval = cur.getFirst();
     }
-    return res;
+    // Ensure we end up in the top right corner.
+    // Simplification will skip this if we already were.
+    curve.addAndSimplify(1.0, 1.0);
+    return curve;
   }
 
   /**
@@ -163,27 +175,28 @@ public class ROC {
     /**
      * Original Iterator
      */
-    private Iterator<DBID> iter;
+    private DBIDIter iter;
 
     /**
      * Constructor
      * 
      * @param iter Iterator for object IDs
      */
-    public SimpleAdapter(Iterator<DBID> iter) {
+    public SimpleAdapter(DBIDIter iter) {
       super();
       this.iter = iter;
     }
 
     @Override
     public boolean hasNext() {
-      return this.iter.hasNext();
+      return this.iter.valid();
     }
 
     @Override
     public DBIDPair next() {
-      DBID id = this.iter.next();
-      return DBIDUtil.newPair(id, id);
+      DBIDPair pair = DBIDUtil.newPair(iter, iter);
+      this.iter.advance();
+      return pair;
     }
 
     @Override
@@ -207,14 +220,14 @@ public class ROC {
     /**
      * Original Iterator
      */
-    private Iterator<DistanceResultPair<D>> iter;
+    private Iterator<? extends DistanceResultPair<D>> iter;
 
     /**
      * Constructor
      * 
      * @param iter Iterator for distance results
      */
-    public DistanceResultAdapter(Iterator<DistanceResultPair<D>> iter) {
+    public DistanceResultAdapter(Iterator<? extends DistanceResultPair<D>> iter) {
       super();
       this.iter = iter;
     }
@@ -250,7 +263,7 @@ public class ROC {
     /**
      * Original Iterator
      */
-    private Iterator<DBID> iter;
+    private DBIDIter iter;
 
     /**
      * Outlier score
@@ -264,18 +277,19 @@ public class ROC {
      */
     public OutlierScoreAdapter(OutlierResult o) {
       super();
-      this.iter = o.getOrdering().iter(o.getScores().getDBIDs());
+      this.iter = o.getOrdering().iter(o.getScores().getDBIDs()).iter();
       this.scores = o.getScores();
     }
 
     @Override
     public boolean hasNext() {
-      return this.iter.hasNext();
+      return this.iter.valid();
     }
 
     @Override
     public DoubleObjPair<DBID> next() {
-      DBID id = this.iter.next();
+      DBID id = this.iter.getDBID();
+      iter.advance();
       return new DoubleObjPair<DBID>(scores.get(id), id);
     }
 
@@ -285,38 +299,6 @@ public class ROC {
     }
   }
 
-  /**
-   * compute the Area Under Curve (difference to y axis) for an arbitrary
-   * polygon
-   * 
-   * @param curve Iterable list of points (x,y)
-   * @return area und curve
-   */
-  public static double computeAUC(Iterable<DoubleDoublePair> curve) {
-    double result = 0.0;
-    Iterator<DoubleDoublePair> iter = curve.iterator();
-    // it doesn't make sense to speak about the "area under a curve" when there
-    // is no curve.
-    if(!iter.hasNext()) {
-      return Double.NaN;
-    }
-    // starting point
-    DoubleDoublePair prev = iter.next();
-    // check there is at least a second point
-    if(!iter.hasNext()) {
-      return Double.NaN;
-    }
-    while(iter.hasNext()) {
-      DoubleDoublePair next = iter.next();
-      // width * height at half way.
-      double width = next.first - prev.first;
-      double meanheight = (next.second + prev.second) / 2;
-      result += width * meanheight;
-      prev = next;
-    }
-    return result;
-  }
-
   /**
    * Compute a ROC curves Area-under-curve for a QueryResult and a Cluster.
    * 
@@ -326,7 +308,7 @@ public class ROC {
    * @param nei Query result
    * @return area under curve
    */
-  public static <D extends Distance<D>> double computeROCAUCDistanceResult(int size, Cluster<?> clus, List<DistanceResultPair<D>> nei) {
+  public static <D extends Distance<D>> double computeROCAUCDistanceResult(int size, Cluster<?> clus, Iterable<? extends DistanceResultPair<D>> nei) {
     // TODO: ensure the collection has efficient "contains".
     return ROC.computeROCAUCDistanceResult(size, clus.getIDs(), nei);
   }
@@ -340,10 +322,10 @@ public class ROC {
    * @param nei Query Result
    * @return area under curve
    */
-  public static <D extends Distance<D>> double computeROCAUCDistanceResult(int size, DBIDs ids, List<DistanceResultPair<D>> nei) {
+  public static <D extends Distance<D>> double computeROCAUCDistanceResult(int size, DBIDs ids, Iterable<? extends DistanceResultPair<D>> nei) {
     // TODO: do not materialize the ROC, but introduce an iterator interface
-    List<DoubleDoublePair> roc = materializeROC(size, DBIDUtil.ensureSet(ids), new DistanceResultAdapter<D>(nei.iterator()));
-    return computeAUC(roc);
+    XYCurve roc = materializeROC(size, DBIDUtil.ensureSet(ids), new DistanceResultAdapter<D>(nei.iterator()));
+    return XYCurve.areaUnderCurve(roc);
   }
 
   /**
@@ -356,7 +338,7 @@ public class ROC {
    */
   public static double computeROCAUCSimple(int size, DBIDs ids, DBIDs nei) {
     // TODO: do not materialize the ROC, but introduce an iterator interface
-    List<DoubleDoublePair> roc = materializeROC(size, DBIDUtil.ensureSet(ids), new SimpleAdapter(nei.iterator()));
-    return computeAUC(roc);
+    XYCurve roc = materializeROC(size, DBIDUtil.ensureSet(ids), new SimpleAdapter(nei.iter()));
+    return XYCurve.areaUnderCurve(roc);
   }
-}
+}
\ No newline at end of file
