@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.preprocessed.preference;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -27,7 +27,6 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.BitSet;
 import java.util.HashMap;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
@@ -42,9 +41,11 @@ import de.lmu.ifi.dbs.elki.database.UpdatableDatabase;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
+import de.lmu.ifi.dbs.elki.database.query.DistanceDBIDResult;
 import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.PrimitiveDistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
@@ -161,9 +162,9 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?, ?>> extends Abs
       // epsilons as string
       RangeQuery<V, DoubleDistance>[] rangeQueries = initRangeQueries(relation, dim);
 
-      for(Iterator<DBID> it = relation.iterDBIDs(); it.hasNext();) {
+      for(DBIDIter it = relation.iterDBIDs(); it.valid(); it.advance()) {
         StringBuffer msg = new StringBuffer();
-        final DBID id = it.next();
+        final DBID id = it.getDBID();
 
         if(logger.isDebugging()) {
           msg.append("\nid = ").append(id);
@@ -174,7 +175,7 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?, ?>> extends Abs
         // determine neighbors in each dimension
         ModifiableDBIDs[] allNeighbors = ClassGenericsUtil.newArrayOfNull(dim, ModifiableDBIDs.class);
         for(int d = 0; d < dim; d++) {
-          List<DistanceResultPair<DoubleDistance>> qrList = rangeQueries[d].getRangeForDBID(id, epsilon[d]);
+          DistanceDBIDResult<DoubleDistance> qrList = rangeQueries[d].getRangeForDBID(id, epsilon[d]);
           allNeighbors[d] = DBIDUtil.newHashSet(qrList.size());
           for(DistanceResultPair<DoubleDistance> qr : qrList) {
             allNeighbors[d].add(qr.getDBID());
@@ -260,8 +261,8 @@ public class DiSHPreferenceVectorIndex<V extends NumberVector<?, ?>> extends Abs
     // database for apriori
     UpdatableDatabase apriori_db = new HashmapDatabase();
     SimpleTypeInformation<?> bitmeta = VectorFieldTypeInformation.get(BitVector.class, dimensionality);
-    for(Iterator<DBID> it = relation.iterDBIDs(); it.hasNext();) {
-      DBID id = it.next();
+    for(DBIDIter it = relation.iterDBIDs(); it.valid(); it.advance()) {
+      DBID id = it.getDBID();
       Bit[] bits = new Bit[dimensionality];
       boolean allFalse = true;
       for(int d = 0; d < dimensionality; d++) {
