@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.metrical.mtreevariants;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -29,6 +29,7 @@ import java.util.List;
 import java.util.Map;
 
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.distance.DistanceUtil;
@@ -378,7 +379,8 @@ public abstract class AbstractMTree<O, D extends Distance<D>, N extends Abstract
     if(node.isLeaf()) {
       for(int i = 0; i < node.getNumEntries(); i++) {
         E p = node.getEntry(i);
-        for(DBID q : ids) {
+        for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+          DBID q = iter.getDBID();
           KNNHeap<D> knns_q = knnLists.get(q);
           D knn_q_maxDist = knns_q.getKNNDistance();
 
@@ -393,7 +395,8 @@ public abstract class AbstractMTree<O, D extends Distance<D>, N extends Abstract
       List<DistanceEntry<D, E>> entries = getSortedEntries(node, ids);
       for(DistanceEntry<D, E> distEntry : entries) {
         D minDist = distEntry.getDistance();
-        for(DBID q : ids) {
+        for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+          DBID q = iter.getDBID();
           KNNHeap<D> knns_q = knnLists.get(q);
           D knn_q_maxDist = knns_q.getKNNDistance();
 
@@ -446,8 +449,8 @@ public abstract class AbstractMTree<O, D extends Distance<D>, N extends Abstract
       E entry = node.getEntry(i);
 
       D minMinDist = getDistanceFactory().infiniteDistance();
-      for(DBID q : ids) {
-        D distance = distanceQuery.distance(entry.getRoutingObjectID(), q);
+      for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+        D distance = distanceQuery.distance(entry.getRoutingObjectID(), iter.getDBID());
         D minDist = entry.getCoveringRadius().compareTo(distance) > 0 ? getDistanceFactory().nullDistance() : distance.minus(entry.getCoveringRadius());
         minMinDist = DistanceUtil.max(minMinDist, minDist);
       }
