@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -27,22 +27,17 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.ObjectOutputStream;
 import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.HashMap;
+import java.util.BitSet;
 import java.util.List;
-import java.util.Map;
 import java.util.Stack;
 
 import de.lmu.ifi.dbs.elki.data.HyperBoundingBox;
+import de.lmu.ifi.dbs.elki.data.ModifiableHyperBoundingBox;
 import de.lmu.ifi.dbs.elki.data.spatial.SpatialComparable;
 import de.lmu.ifi.dbs.elki.data.spatial.SpatialUtil;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
-import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.index.tree.BreadthFirstEnumeration;
-import de.lmu.ifi.dbs.elki.index.tree.DistanceEntry;
 import de.lmu.ifi.dbs.elki.index.tree.IndexTreePath;
 import de.lmu.ifi.dbs.elki.index.tree.LeafEntry;
 import de.lmu.ifi.dbs.elki.index.tree.TreeIndexHeader;
@@ -51,16 +46,17 @@ import de.lmu.ifi.dbs.elki.index.tree.spatial.SpatialDirectoryEntry;
 import de.lmu.ifi.dbs.elki.index.tree.spatial.SpatialEntry;
 import de.lmu.ifi.dbs.elki.index.tree.spatial.SpatialIndexTree;
 import de.lmu.ifi.dbs.elki.index.tree.spatial.SpatialPointLeafEntry;
-import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.bulk.BulkSplit;
-import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.util.Enlargement;
-import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.util.InsertionStrategy;
-import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.util.LeastOverlapInsertionStrategy;
-import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.util.SplitStrategy;
-import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.util.TopologicalSplitter;
+import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.strategies.bulk.BulkSplit;
+import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.strategies.insert.InsertionStrategy;
+import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.strategies.insert.LeastOverlapInsertionStrategy;
+import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.strategies.overflow.LimitedReinsertOverflowTreatment;
+import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.strategies.overflow.OverflowTreatment;
+import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.strategies.split.SplitStrategy;
+import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.strategies.split.TopologicalSplitter;
+import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.util.NodeArrayAdapter;
 import de.lmu.ifi.dbs.elki.persistent.PageFile;
 import de.lmu.ifi.dbs.elki.persistent.PageFileUtil;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
-import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 
 /**
  * Abstract superclass for index structures based on a R*-Tree.
@@ -72,10 +68,10 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
  * 
  * @apiviz.landmark
  * @apiviz.has AbstractRStarTreeNode oneway - - contains
- * @apiviz.uses Enlargement
  * @apiviz.composedOf BulkSplit
  * @apiviz.composedOf SplitStrategy
  * @apiviz.composedOf InsertionStrategy
+ * @apiviz.composedOf OverflowTreatment
  * 
  * @param <N> Node type
  * @param <E> Entry type
@@ -86,13 +82,6 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
    */
   protected final static boolean extraIntegrityChecks = false;
 
-  /**
-   * Contains a boolean for each level of this R*-Tree that indicates if there
-   * was already a reinsert operation in this level during the current insert /
-   * delete operation.
-   */
-  protected final Map<Integer, Boolean> reinsertions = new HashMap<Integer, Boolean>();
-
   /**
    * The height of this R*-Tree.
    */
@@ -116,32 +105,87 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
   /**
    * The split strategy
    */
-  protected SplitStrategy<? super E> nodeSplitter = new TopologicalSplitter();
+  protected SplitStrategy nodeSplitter = TopologicalSplitter.STATIC;
 
   /**
    * The insertion strategy to use
    */
-  protected final InsertionStrategy insertionStrategy;
+  protected InsertionStrategy insertionStrategy = LeastOverlapInsertionStrategy.STATIC;
+
+  /**
+   * Overflow treatment
+   */
+  protected OverflowTreatment overflowTreatment = LimitedReinsertOverflowTreatment.RSTAR_OVERFLOW;
+
+  /**
+   * Relative minimum fill
+   */
+  protected double relativeMinFill = 0.4;
 
   /**
    * Constructor
    * 
    * @param pagefile Page file
-   * @param bulkSplitter bulk load strategy
-   * @param insertionStrategy the strategy for finding the insertion candidate.
    */
-  public AbstractRStarTree(PageFile<N> pagefile, BulkSplit bulkSplitter, InsertionStrategy insertionStrategy) {
+  public AbstractRStarTree(PageFile<N> pagefile) {
     super(pagefile);
+  }
+
+  /**
+   * Set the bulk loading strategy
+   * 
+   * @param bulkSplitter Bulk loading strategy
+   */
+  public void setBulkStrategy(BulkSplit bulkSplitter) {
     this.bulkSplitter = bulkSplitter;
+  }
+
+  /**
+   * Set the node splitting strategy.
+   * 
+   * @param nodeSplitter the split strategy to set
+   */
+  public void setNodeSplitStrategy(SplitStrategy nodeSplitter) {
+    if(nodeSplitter != null) {
+      this.nodeSplitter = nodeSplitter;
+    }
+    else {
+      getLogger().warning("Ignoring setNodeSplitStrategy(null)");
+    }
+  }
+
+  /**
+   * Set insertion strategy
+   * 
+   * @param insertionStrategy the insertion strategy to set
+   */
+  public void setInsertionStrategy(InsertionStrategy insertionStrategy) {
     if(insertionStrategy != null) {
       this.insertionStrategy = insertionStrategy;
     }
     else {
-      getLogger().warning("No insertion strategy given - falling back to " + LeastOverlapInsertionStrategy.class.getSimpleName());
-      this.insertionStrategy = new LeastOverlapInsertionStrategy();
+      getLogger().warning("Ignoring setInsertionStrategy(null)");
     }
   }
 
+  /**
+   * Set the overflow treatment strategy.
+   * 
+   * @param overflowTreatment overflow treatment strategy
+   */
+  public void setOverflowTreatment(OverflowTreatment overflowTreatment) {
+    this.overflowTreatment = overflowTreatment;
+  }
+
+  /**
+   * Set the relative minimum fill. (Only supported before the tree was used!)
+   * 
+   * @param relative Relative minimum fill
+   */
+  public void setMinimumFill(double relative) {
+    this.relativeMinFill = relative;
+  }
+
   /**
    * Returns the path to the leaf entry in the specified subtree that represents
    * the data object with the specified mbr and id.
@@ -152,11 +196,11 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
    * @return the path to the leaf entry of the specified subtree that represents
    *         the data object with the specified mbr and id
    */
-  protected IndexTreePath<E> findPathToObject(IndexTreePath<E> subtree, SpatialComparable mbr, DBID id) {
+  protected IndexTreePath<E> findPathToObject(IndexTreePath<E> subtree, SpatialComparable mbr, DBIDRef id) {
     N node = getNode(subtree.getLastPathComponent().getEntry());
     if(node.isLeaf()) {
       for(int i = 0; i < node.getNumEntries(); i++) {
-        if(((LeafEntry) node.getEntry(i)).getDBID().equals(id)) {
+        if(((LeafEntry) node.getEntry(i)).getDBID().sameDBID(id)) {
           return subtree.pathByAddingChild(new TreeIndexPathComponent<E>(node.getEntry(i), i));
         }
       }
@@ -181,7 +225,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     if(!initialized) {
       initialize(leaf);
     }
-    reinsertions.clear();
+    overflowTreatment.reinitialize();
 
     preInsert(leaf);
     insertLeafEntry(leaf);
@@ -200,7 +244,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     IndexTreePath<E> subtree = choosePath(getRootPath(), entry, 1);
 
     if(getLogger().isDebugging()) {
-      getLogger().debugFine("insertion-subtree " + subtree + "\n");
+      getLogger().debugFine("insertion-subtree " + subtree);
     }
 
     N parent = getNode(subtree.getLastPathComponent().getEntry());
@@ -257,7 +301,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
       N node = stack.pop();
       if(node.isLeaf()) {
         for(int i = 0; i < node.getNumEntries(); i++) {
-          reinsertions.clear();
+          overflowTreatment.reinitialize(); // Intended?
           this.insertLeafEntry(node.getEntry(i));
         }
       }
@@ -315,7 +359,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
       int cap = 0;
       ByteArrayOutputStream baos = new ByteArrayOutputStream();
       ObjectOutputStream oos = new ObjectOutputStream(baos);
-      HyperBoundingBox hb = new HyperBoundingBox(new double[exampleLeaf.getDimensionality()], new double[exampleLeaf.getDimensionality()]);
+      ModifiableHyperBoundingBox hb = new ModifiableHyperBoundingBox(new double[exampleLeaf.getDimensionality()], new double[exampleLeaf.getDimensionality()]);
       SpatialDirectoryEntry sl = new SpatialDirectoryEntry(0, hb);
       while(baos.size() <= getPageSize()) {
         sl.writeExternal(oos);
@@ -337,7 +381,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     }
 
     // minimum entries per directory node
-    dirMinimum = (int) Math.round((dirCapacity - 1) * 0.4);
+    dirMinimum = (int) Math.round((dirCapacity - 1) * relativeMinFill);
     if(dirMinimum < 2) {
       dirMinimum = 2;
     }
@@ -351,7 +395,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     }
 
     // minimum entries per leaf node
-    leafMinimum = (int) Math.round((leafCapacity - 1) * 0.4);
+    leafMinimum = (int) Math.round((leafCapacity - 1) * relativeMinFill);
     if(leafMinimum < 2) {
       leafMinimum = 2;
     }
@@ -376,33 +420,31 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
    * @param objects the objects to be inserted
    * @return the array of leaf nodes containing the objects
    */
-  protected List<N> createBulkLeafNodes(List<E> objects) {
+  protected List<E> createBulkLeafNodes(List<E> objects) {
     int minEntries = leafMinimum;
     int maxEntries = leafCapacity - 1;
-  
-    ArrayList<N> result = new ArrayList<N>();
+
+    ArrayList<E> result = new ArrayList<E>();
     List<List<E>> partitions = bulkSplitter.partition(objects, minEntries, maxEntries);
-  
+
     for(List<E> partition : partitions) {
       // create leaf node
       N leafNode = createNewLeafNode();
-      result.add(leafNode);
-  
+
       // insert data
       for(E o : partition) {
         leafNode.addLeafEntry(o);
       }
-  
       // write to file
       writeNode(leafNode);
-  
+
+      result.add(createNewDirectoryEntry(leafNode));
+
       if(getLogger().isDebugging()) {
-        StringBuffer msg = new StringBuffer();
-        msg.append("pageNo ").append(leafNode.getPageID()).append("\n");
-        getLogger().debugFine(msg.toString());
+        getLogger().debugFine("Created leaf page "+leafNode.getPageID());
       }
     }
-  
+
     if(getLogger().isDebugging()) {
       getLogger().debugFine("numDataPages = " + result.size());
     }
@@ -440,13 +482,6 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
    */
   abstract protected int computeHeight();
 
-  /**
-   * Clears the reinsertions.
-   */
-  protected void clearReinsertions() {
-    reinsertions.clear();
-  }
-
   /**
    * Returns true if in the specified node an overflow occurred, false
    * otherwise.
@@ -486,7 +521,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
   protected IndexTreePath<E> createNewRoot(final N oldRoot, final N newNode) {
     N root = createNewDirectoryNode();
     writeNode(root);
-  
+
     // switch the ids
     oldRoot.setPageID(root.getPageID());
     if(!oldRoot.isLeaf()) {
@@ -495,24 +530,24 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
         writeNode(node);
       }
     }
-  
+
     root.setPageID(getRootID());
     E oldRootEntry = createNewDirectoryEntry(oldRoot);
     E newNodeEntry = createNewDirectoryEntry(newNode);
     root.addDirectoryEntry(oldRootEntry);
     root.addDirectoryEntry(newNodeEntry);
-  
+
     writeNode(root);
     writeNode(oldRoot);
     writeNode(newNode);
     if(getLogger().isDebugging()) {
       String msg = "Create new Root: ID=" + root.getPageID();
-      msg += "\nchild1 " + oldRoot + " " + new HyperBoundingBox(oldRoot) + " " + new HyperBoundingBox(oldRootEntry);
-      msg += "\nchild2 " + newNode + " " + new HyperBoundingBox(newNode) + " " + new HyperBoundingBox(newNodeEntry);
+      msg += "\nchild1 " + oldRoot + " " + new HyperBoundingBox(oldRootEntry);
+      msg += "\nchild2 " + newNode + " " + new HyperBoundingBox(newNodeEntry);
       msg += "\n";
       getLogger().debugFine(msg);
     }
-  
+
     return new IndexTreePath<E>(new TreeIndexPathComponent<E>(getRootEntry(), null));
   }
 
@@ -591,10 +626,11 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     }
 
     N childNode = getNode(node.getEntry(0));
+    int num = insertionStrategy.choose(node, NodeArrayAdapter.STATIC, mbr, height, subtree.getPathCount());
+    TreeIndexPathComponent<E> comp = new TreeIndexPathComponent<E>(node.getEntry(num), num);
     // children are leafs
     if(childNode.isLeaf()) {
       if(height - subtree.getPathCount() == level) {
-        TreeIndexPathComponent<E> comp = insertionStrategy.findInsertChild(node, mbr);
         return subtree.pathByAddingChild(comp);
       }
       else {
@@ -603,7 +639,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     }
     // children are directory nodes
     else {
-      IndexTreePath<E> newSubtree = subtree.pathByAddingChild(getLeastEnlargement(node, mbr));
+      IndexTreePath<E> newSubtree = subtree.pathByAddingChild(comp);
       // desired level is reached
       if(height - subtree.getPathCount() == level) {
         return newSubtree;
@@ -614,34 +650,6 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     }
   }
 
-  /**
-   * Returns the path information of the entry of the specified node with the
-   * least enlargement if the given mbr would be inserted into.
-   * 
-   * @param node the node which children have to be tested
-   * @param mbr the mbr of the node to be inserted
-   * @return the path information of the entry with the least enlargement if the
-   *         given mbr would be inserted into
-   */
-  private TreeIndexPathComponent<E> getLeastEnlargement(N node, SpatialComparable mbr) {
-    Enlargement<E> min = null;
-
-    for(int i = 0; i < node.getNumEntries(); i++) {
-      E entry = node.getEntry(i);
-      double volume = SpatialUtil.volume(entry);
-      HyperBoundingBox newMBR = SpatialUtil.union(entry, mbr);
-      double inc = SpatialUtil.volume(newMBR) - volume;
-      Enlargement<E> enlargement = new Enlargement<E>(new TreeIndexPathComponent<E>(entry, i), volume, inc, 0);
-
-      if(min == null || min.compareTo(enlargement) > 0) {
-        min = enlargement;
-      }
-    }
-
-    assert min != null;
-    return min.getPathComponent();
-  }
-
   /**
    * Treatment of overflow in the specified node: if the node is not the root
    * node and this is the first call of overflowTreatment in the given level
@@ -654,23 +662,10 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
    *         reinsertion
    */
   private N overflowTreatment(N node, IndexTreePath<E> path) {
-    int level = height - path.getPathCount() + 1;
-    Boolean reInsert = reinsertions.get(level);
-
-    // there was still no reinsert operation at this level
-    if(node.getPageID() != 0 && (reInsert == null || !reInsert)) {
-      reinsertions.put(level, true);
-      if(getLogger().isDebugging()) {
-        getLogger().debugFine("REINSERT " + reinsertions + "\n");
-      }
-      reInsert(node, level, path);
+    if(overflowTreatment.handleOverflow(this, node, path)) {
       return null;
     }
-
-    // there was already a reinsert operation at this level
-    else {
-      return split(node);
-    }
+    return split(node);
   }
 
   /**
@@ -682,7 +677,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
   private N split(N node) {
     // choose the split dimension and the split point
     int minimum = node.isLeaf() ? leafMinimum : dirMinimum;
-    Pair<List<E>, List<E>> split = nodeSplitter.split(node.getEntries(), minimum);
+    BitSet split = nodeSplitter.split(node, NodeArrayAdapter.STATIC, minimum);
 
     // New node
     final N newNode;
@@ -693,22 +688,12 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
       newNode = createNewDirectoryNode();
     }
     // do the split
-    node.deleteAllEntries();
-    node.splitTo(newNode, split.first, split.second);
+    node.splitByMask(newNode, split);
 
     // write changes to file
     writeNode(node);
     writeNode(newNode);
 
-    // if(getLogger().isDebugging()) {
-    // StringBuffer msg = new StringBuffer();
-    // msg.append("Split Node ").append(node.getPageID()).append(" (").append(getClass()).append(")\n");
-    // msg.append("      splitAxis ").append(split.getSplitAxis()).append("\n");
-    // msg.append("      splitPoint ").append(split.getSplitPoint()).append("\n");
-    // msg.append("      newNode ").append(newNode.getPageID()).append("\n");
-    // getLogger().debugFine(msg.toString());
-    // }
-
     return newNode;
   }
 
@@ -716,29 +701,20 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
    * Reinserts the specified node at the specified level.
    * 
    * @param node the node to be reinserted
-   * @param level the level of the node
    * @param path the path to the node
+   * @param offs the nodes indexes to reinsert
    */
-  @SuppressWarnings("unchecked")
-  protected void reInsert(N node, int level, IndexTreePath<E> path) {
-    EuclideanDistanceFunction distFunction = EuclideanDistanceFunction.STATIC;
-    DistanceEntry<DoubleDistance, E>[] reInsertEntries = new DistanceEntry[node.getNumEntries()];
+  public void reInsert(N node, IndexTreePath<E> path, int[] offs) {
+    final int level = height - (path.getPathCount() - 1);
 
-    // compute the center distances of entries to the node and sort it
-    // in decreasing order to their distances
-    for(int i = 0; i < node.getNumEntries(); i++) {
-      E entry = node.getEntry(i);
-      DoubleDistance dist = distFunction.centerDistance(node, entry);
-      reInsertEntries[i] = new DistanceEntry<DoubleDistance, E>(entry, dist, i);
+    BitSet remove = new BitSet();
+    List<E> reInsertEntries = new ArrayList<E>(offs.length);
+    for(int i = 0; i < offs.length; i++) {
+      reInsertEntries.add(node.getEntry(offs[i]));
+      remove.set(offs[i]);
     }
-    Arrays.sort(reInsertEntries, Collections.reverseOrder());
-
-    // define, how many entries will be reinserted
-    int start = (int) (0.3 * node.getNumEntries());
-
-    // initialize the reinsertion operation: move the remaining entries
-    // forward
-    node.initReInsert(start, reInsertEntries);
+    // Remove the entries we reinsert
+    node.removeMask(remove);
     writeNode(node);
 
     // and adapt the mbrs
@@ -747,26 +723,30 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     while(childPath.getParentPath() != null) {
       N parent = getNode(childPath.getParentPath().getLastPathComponent().getEntry());
       int indexOfChild = childPath.getLastPathComponent().getIndex();
-      child.adjustEntry(parent.getEntry(indexOfChild));
-      writeNode(parent);
-      childPath = childPath.getParentPath();
-      child = parent;
+      if(child.adjustEntry(parent.getEntry(indexOfChild))) {
+        writeNode(parent);
+        childPath = childPath.getParentPath();
+        child = parent;
+      }
+      else {
+        break;
+        // TODO: stop writing when MBR didn't change!
+      }
     }
 
     // reinsert the first entries
-    for(int i = 0; i < start; i++) {
-      DistanceEntry<DoubleDistance, E> re = reInsertEntries[i];
+    for(E entry : reInsertEntries) {
       if(node.isLeaf()) {
         if(getLogger().isDebugging()) {
-          getLogger().debugFine("reinsert " + re.getEntry());
+          getLogger().debug("reinsert " + entry);
         }
-        insertLeafEntry(re.getEntry());
+        insertLeafEntry(entry);
       }
       else {
         if(getLogger().isDebugging()) {
-          getLogger().debugFine("reinsert " + re.getEntry() + " at " + level);
+          getLogger().debug("reinsert " + entry + " at " + level);
         }
-        insertDirectoryEntry(re.getEntry(), level);
+        insertDirectoryEntry(entry, level);
       }
     }
   }
@@ -778,7 +758,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
    */
   protected void adjustTree(IndexTreePath<E> subtree) {
     if(getLogger().isDebugging()) {
-      getLogger().debugFine("Adjust tree " + subtree + "\n");
+      getLogger().debugFine("Adjust tree " + subtree);
     }
 
     // get the root of the subtree
@@ -826,11 +806,13 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
       if(!isRoot(node)) {
         N parent = getNode(subtree.getParentPath().getLastPathComponent().getEntry());
         E entry = parent.getEntry(subtree.getLastPathComponent().getIndex());
-        lastInsertedEntry = node.adjustEntryIncremental(entry, lastInsertedEntry);
-        // node.adjustEntry(parent.getEntry(index));
-        // write changes in parent to file
-        writeNode(parent);
-        adjustTree(subtree.getParentPath());
+        boolean changed = node.adjustEntryIncremental(entry, lastInsertedEntry);
+        if(changed) {
+          // node.adjustEntry(parent.getEntry(index));
+          // write changes in parent to file
+          writeNode(parent);
+          adjustTree(subtree.getParentPath());
+        }
       }
       // root level is reached
       else {
@@ -896,12 +878,12 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
   @Override
   public final List<E> getLeaves() {
     List<E> result = new ArrayList<E>();
-  
+
     if(height == 1) {
       result.add(getRootEntry());
       return result;
     }
-  
+
     getLeafNodes(getRoot(), result, height);
     return result;
   }
@@ -949,11 +931,11 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     int leafNodes = 0;
     int objects = 0;
     int levels = 0;
-  
+
     if(initialized) {
       N node = getRoot();
-      int dim = node.getDimensionality();
-  
+      int dim = getRootEntry().getDimensionality();
+
       while(!node.isLeaf()) {
         if(node.getNumEntries() > 0) {
           E entry = node.getEntry(0);
@@ -961,7 +943,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
           levels++;
         }
       }
-  
+
       BreadthFirstEnumeration<N, E> enumeration = new BreadthFirstEnumeration<N, E>(this, getRootPath());
       while(enumeration.hasMoreElements()) {
         IndexTreePath<E> indexPath = enumeration.nextElement();
@@ -988,7 +970,7 @@ public abstract class AbstractRStarTree<N extends AbstractRStarTreeNode<N, E>, E
     else {
       result.append(getClass().getName()).append(" is empty!\n");
     }
-  
+
     return result.toString();
   }
 }
\ No newline at end of file
