@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.deliclu;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -28,6 +28,7 @@ import java.util.List;
 
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.SpatialDistanceQuery;
@@ -39,9 +40,7 @@ import de.lmu.ifi.dbs.elki.index.KNNIndex;
 import de.lmu.ifi.dbs.elki.index.RangeIndex;
 import de.lmu.ifi.dbs.elki.index.tree.IndexTreePath;
 import de.lmu.ifi.dbs.elki.index.tree.TreeIndexPathComponent;
-import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.bulk.BulkSplit;
 import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.query.RStarTreeUtil;
-import de.lmu.ifi.dbs.elki.index.tree.spatial.rstarvariants.util.InsertionStrategy;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.persistent.PageFile;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
@@ -64,11 +63,9 @@ public class DeLiCluTreeIndex<O extends NumberVector<?, ?>> extends DeLiCluTree
    * 
    * @param relation Relation to index
    * @param pagefile Page file
-   * @param bulkSplitter bulk load strategy
-   * @param insertionStrategy the strategy to find the insertion child
    */
-  public DeLiCluTreeIndex(Relation<O> relation, PageFile<DeLiCluNode> pagefile, BulkSplit bulkSplitter, InsertionStrategy insertionStrategy) {
-    super(pagefile, bulkSplitter, insertionStrategy);
+  public DeLiCluTreeIndex(Relation<O> relation, PageFile<DeLiCluNode> pagefile) {
+    super(pagefile);
     this.relation = relation;
     this.initialize();
   }
@@ -154,14 +151,14 @@ public class DeLiCluTreeIndex<O extends NumberVector<?, ?>> extends DeLiCluTree
     // Make an example leaf
     if(canBulkLoad()) {
       List<DeLiCluEntry> leafs = new ArrayList<DeLiCluEntry>(ids.size());
-      for(DBID id : ids) {
-        leafs.add(createNewLeafEntry(id));
+      for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+        leafs.add(createNewLeafEntry(iter.getDBID()));
       }
       bulkLoad(leafs);
     }
     else {
-      for(DBID id : ids) {
-        insert(id);
+      for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+        insert(iter.getDBID());
       }
     }
 
@@ -188,8 +185,8 @@ public class DeLiCluTreeIndex<O extends NumberVector<?, ?>> extends DeLiCluTree
 
   @Override
   public void deleteAll(DBIDs ids) {
-    for(DBID id : ids) {
-      delete(id);
+    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      delete(iter.getDBID());
     }
   }
 
