@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.math.linearalgebra;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -733,7 +733,7 @@ public class LinearEquationSystem {
     if(value < 10) {
       return 1;
     }
-    return (int) (Math.log(value) / Math.log(10) + 1);
+    return (int) Math.log10(value) + 1;
   }
 
   /**
