@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -45,6 +45,7 @@ import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.ConvertToStringView;
@@ -181,8 +182,8 @@ public final class DatabaseUtil {
     for(int d = 1; d <= centroid.getDimensionality(); d++) {
       double mu = centroid.doubleValue(d);
 
-      for(Iterator<DBID> it = database.iterDBIDs(); it.hasNext();) {
-        NumberVector<?, ?> o = database.get(it.next());
+      for(DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
+        NumberVector<?, ?> o = database.get(it);
         double diff = o.doubleValue(d) - mu;
         variances[d - 1] += diff * diff;
       }
@@ -220,8 +221,8 @@ public final class DatabaseUtil {
     for(int d = 1; d <= centroid.getDimensionality(); d++) {
       double mu = centroid.doubleValue(d);
 
-      for(DBID id : ids) {
-        NumberVector<?, ?> o = database.get(id);
+      for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+        NumberVector<?, ?> o = database.get(iter);
         double diff = o.doubleValue(d) - mu;
         variances[d - 1] += diff * diff;
       }
@@ -247,17 +248,17 @@ public final class DatabaseUtil {
       mins[i] = Double.MAX_VALUE;
       maxs[i] = -Double.MAX_VALUE;
     }
-    for(DBID it : database.iterDBIDs()) {
-      NV o = database.get(it);
+    for(DBIDIter iditer = database.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      final NV o = database.get(iditer);
       for(int d = 0; d < dim; d++) {
-        double v = o.doubleValue(d + 1);
+        final double v = o.doubleValue(d + 1);
         mins[d] = Math.min(mins[d], v);
         maxs[d] = Math.max(maxs[d], v);
       }
     }
     NV factory = assumeVectorField(database).getFactory();
-    NV min = factory.newInstance(mins);
-    NV max = factory.newInstance(maxs);
+    NV min = factory.newNumberVector(mins);
+    NV max = factory.newNumberVector(maxs);
     return new Pair<NV, NV>(min, max);
   }
 
@@ -300,8 +301,8 @@ public final class DatabaseUtil {
   public static <V extends NumberVector<?, ?>> double exactMedian(Relation<V> relation, DBIDs ids, int dimension) {
     final double[] vals = new double[ids.size()];
     int i = 0;
-    for(DBID id : ids) {
-      vals[i] = relation.get(id).doubleValue(dimension);
+    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      vals[i] = relation.get(iter).doubleValue(dimension);
       i++;
     }
     Arrays.sort(vals);
@@ -316,7 +317,7 @@ public final class DatabaseUtil {
   }
 
   /**
-   * Guess a potentially label-like representation.
+   * Guess a potentially label-like representation, preferring class labels.
    * 
    * @param database
    * @return string representation
@@ -398,8 +399,8 @@ public final class DatabaseUtil {
    */
   public static SortedSet<ClassLabel> getClassLabels(Relation<? extends ClassLabel> database) {
     SortedSet<ClassLabel> labels = new TreeSet<ClassLabel>();
-    for(Iterator<DBID> iter = database.iterDBIDs(); iter.hasNext();) {
-      labels.add(database.get(iter.next()));
+    for(DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
+      labels.add(database.get(it));
     }
     return labels;
   }
@@ -425,10 +426,7 @@ public final class DatabaseUtil {
    */
   @SuppressWarnings("unchecked")
   public static <O> Class<? extends O> guessObjectClass(Relation<O> database) {
-    for(DBID id : database.iterDBIDs()) {
-      return (Class<? extends O>) database.get(id).getClass();
-    }
-    return null;
+    return (Class<? extends O>) database.get(database.iterDBIDs()).getClass();
   }
 
   /**
@@ -445,16 +443,17 @@ public final class DatabaseUtil {
    */
   public static <O> Class<?> getBaseObjectClassExpensive(Relation<O> database) {
     List<Class<?>> candidates = new ArrayList<Class<?>>();
-    Iterator<DBID> iditer = database.iterDBIDs();
+    DBIDIter iditer = database.iterDBIDs();
     // empty database?!
-    if(!iditer.hasNext()) {
+    if(!iditer.valid()) {
       return null;
     }
     // put first class into result set.
-    candidates.add(database.get(iditer.next()).getClass());
+    candidates.add(database.get(iditer).getClass());
+    iditer.advance();
     // other objects
-    while(iditer.hasNext()) {
-      Class<?> newcls = database.get(iditer.next()).getClass();
+    for(; iditer.valid(); iditer.advance()) {
+      Class<?> newcls = database.get(iditer).getClass();
       // validate all candidates
       Iterator<Class<?>> ci = candidates.iterator();
       while(ci.hasNext()) {
@@ -509,9 +508,9 @@ public final class DatabaseUtil {
       return DBIDUtil.newArray();
     }
     ArrayModifiableDBIDs ret = DBIDUtil.newArray();
-    for(DBID objid : relation.iterDBIDs()) {
-      if(name_pattern.matcher(relation.get(objid)).matches()) {
-        ret.add(objid);
+    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      if(name_pattern.matcher(relation.get(iditer)).find()) {
+        ret.add(iditer);
       }
     }
     return ret;
@@ -556,7 +555,7 @@ public final class DatabaseUtil {
     /**
      * The real iterator.
      */
-    final Iterator<DBID> iter;
+    final DBIDIter iter;
 
     /**
      * The database we use
@@ -569,7 +568,7 @@ public final class DatabaseUtil {
      * @param iter Original iterator.
      * @param database Database
      */
-    public RelationObjectIterator(Iterator<DBID> iter, Relation<? extends O> database) {
+    public RelationObjectIterator(DBIDIter iter, Relation<? extends O> database) {
       super();
       this.iter = iter;
       this.database = database;
@@ -588,18 +587,19 @@ public final class DatabaseUtil {
 
     @Override
     public boolean hasNext() {
-      return iter.hasNext();
+      return iter.valid();
     }
 
     @Override
     public O next() {
-      DBID id = iter.next();
-      return database.get(id);
+      O ret = database.get(iter);
+      iter.advance();
+      return ret;
     }
 
     @Override
     public void remove() {
-      iter.remove();
+      throw new UnsupportedOperationException();
     }
   }
 
