@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.utilities;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -34,7 +34,6 @@ import java.util.List;
 import java.util.Locale;
 
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
-import de.lmu.ifi.dbs.elki.math.linearalgebra.MatrixLike;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 
 /**
@@ -46,6 +45,11 @@ public final class FormatUtil {
    */
   public static final NumberFormat NF = NumberFormat.getInstance(Locale.US);
 
+  /**
+   * Number Formatter (0 digits) for output purposes.
+   */
+  public static final NumberFormat NF0 = NumberFormat.getInstance(Locale.US);
+
   /**
    * Number Formatter (2 digits) for output purposes.
    */
@@ -70,6 +74,9 @@ public final class FormatUtil {
     NF.setMinimumFractionDigits(0);
     NF.setMaximumFractionDigits(10);
     NF.setGroupingUsed(false);
+    NF0.setMinimumFractionDigits(0);
+    NF0.setMaximumFractionDigits(0);
+    NF0.setGroupingUsed(false);
     NF2.setMinimumFractionDigits(2);
     NF2.setMaximumFractionDigits(2);
     NF2.setGroupingUsed(false);
@@ -595,7 +602,7 @@ public final class FormatUtil {
    * @return a string representation of this matrix
    */
   // TODO: in use?
-  public static String format(MatrixLike<?> m, int w, int d) {
+  public static String format(Matrix m, int w, int d) {
     DecimalFormat format = new DecimalFormat();
     format.setDecimalFormatSymbols(new DecimalFormatSymbols(Locale.US));
     format.setMinimumIntegerDigits(1);
@@ -623,6 +630,39 @@ public final class FormatUtil {
     return msg.toString();
   }
 
+  /**
+   * Returns a string representation of this matrix.
+   * 
+   * @param w column width
+   * @param d number of digits after the decimal
+   * @return a string representation of this matrix
+   */
+  // TODO: in use?
+  public static String format(Vector v, int w, int d) {
+    DecimalFormat format = new DecimalFormat();
+    format.setDecimalFormatSymbols(new DecimalFormatSymbols(Locale.US));
+    format.setMinimumIntegerDigits(1);
+    format.setMaximumFractionDigits(d);
+    format.setMinimumFractionDigits(d);
+    format.setGroupingUsed(false);
+
+    int width = w + 1;
+    StringBuffer msg = new StringBuffer();
+    msg.append("\n"); // start on new line.
+    for(int i = 0; i < v.getDimensionality(); i++) {
+      String s = format.format(v.get(i)); // format the number
+      int padding = Math.max(1, width - s.length()); // At _least_ 1
+      // space
+      for(int k = 0; k < padding; k++) {
+        msg.append(' ');
+      }
+      msg.append(s);
+    }
+    // msg.append("\n");
+
+    return msg.toString();
+  }
+
   /**
    * Returns a string representation of this matrix. In each line the specified
    * String <code>pre</code> is prefixed.
@@ -630,7 +670,7 @@ public final class FormatUtil {
    * @param pre the prefix of each line
    * @return a string representation of this matrix
    */
-  public static String format(MatrixLike<?> m, String pre) {
+  public static String format(Matrix m, String pre) {
     StringBuffer output = new StringBuffer();
     output.append(pre).append("[\n").append(pre);
     for(int i = 0; i < m.getRowDimensionality(); i++) {
@@ -717,6 +757,27 @@ public final class FormatUtil {
     return format(m, FormatUtil.NF8);
   }
 
+  /**
+   * Returns a string representation of this matrix. In each line the specified
+   * String <code>pre</code> is prefixed.
+   * 
+   * @param pre the prefix of each line
+   * @return a string representation of this matrix
+   */
+  public static String format(Vector v, String pre) {
+    StringBuffer output = new StringBuffer();
+    output.append(pre).append("[\n").append(pre);
+    for(int j = 0; j < v.getDimensionality(); j++) {
+      output.append(" ").append(v.get(j));
+      if(j < v.getDimensionality() - 1) {
+        output.append(",");
+      }
+    }
+    output.append("]\n").append(pre);
+
+    return (output.toString());
+  }
+
   /**
    * Returns a string representation of this matrix. In each line the specified
    * String <code>pre<\code> is prefixed.
@@ -725,7 +786,7 @@ public final class FormatUtil {
    * @param pre the prefix of each line
    * @return a string representation of this matrix
    */
-  public static String format(MatrixLike<?> m, String pre, NumberFormat nf) {
+  public static String format(Matrix m, String pre, NumberFormat nf) {
     if(nf == null) {
       return FormatUtil.format(m, pre);
     }
