@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -40,7 +40,6 @@ import de.lmu.ifi.dbs.elki.visualization.visualizers.VisFactory;
  * @apiviz.has SVGPlot
  * @apiviz.has VisFactory
  * @apiviz.has Projection oneway - 0:1
- * @apiviz.has Visualization oneway
  */
 public class VisualizationTask extends AnyMap<String> implements Cloneable, Result, Comparable<VisualizationTask> {
   /**
@@ -94,6 +93,11 @@ public class VisualizationTask extends AnyMap<String> implements Cloneable, Resu
    */
   public final static String META_NOEXPORT = "no-export";
 
+  /**
+   * Flag to signal the visualizer should not be embed. Type: Boolean
+   */
+  public final static String META_NOEMBED = "no-embed";
+
   /**
    * Flag to signal default visibility of a visualizer. Type: Boolean
    */
@@ -104,6 +108,11 @@ public class VisualizationTask extends AnyMap<String> implements Cloneable, Resu
    */
   public static final String META_TOOL = "tool";
 
+  /**
+   * Indicate whether this task has options.
+   */
+  public static final String META_HAS_OPTIONS = "has-options";
+
   /**
    * Background layer
    */
@@ -324,7 +333,7 @@ public class VisualizationTask extends AnyMap<String> implements Cloneable, Resu
     Integer level1 = this.get(VisualizationTask.META_LEVEL, Integer.class);
     Integer level2 = other.get(VisualizationTask.META_LEVEL, Integer.class);
     if(level1 != null && level2 != null && level1 != level2) {
-      return level2 - level1;
+      return level1 - level2;
     }
     // sort by name otherwise.
     String name1 = this.getShortName();
