@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.gui.overview;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -302,6 +302,10 @@ public class RectangleArranger<T> {
   }
 
   protected void splitRow(int bestey, double besthi) {
+    assert(bestey < heights.size());
+    if (heights.get(bestey) - besthi <= Double.MIN_NORMAL) {
+      return;
+    }
     logger.debugFine("Split row " + bestey);
     heights.add(bestey + 1, besthi);
     heights.set(bestey, heights.get(bestey) - besthi);
@@ -310,6 +314,10 @@ public class RectangleArranger<T> {
   }
 
   protected void splitCol(int bestex, double bestwi) {
+    assert(bestex < widths.size());
+    if (widths.get(bestex) - bestwi <= Double.MIN_NORMAL) {
+      return;
+    }
     final int rows = heights.size();
     logger.debugFine("Split column " + bestex);
     widths.add(bestex + 1, bestwi);
@@ -371,7 +379,7 @@ public class RectangleArranger<T> {
     {
       double wsum = 0.0;
       for(int x = 0; x < cols; x++) {
-        assert (widths.get(x) > 0) : "Negative width: "+widths.get(x);
+        assert (widths.get(x) > 0) : "Non-positive width: "+widths.get(x);
         wsum += widths.get(x);
       }
       assert (Math.abs(wsum - twidth) < 1E-10);
@@ -379,7 +387,7 @@ public class RectangleArranger<T> {
     {
       double hsum = 0.0;
       for(int y = 0; y < rows; y++) {
-        assert (heights.get(y) > 0) : "Negative height: "+heights.get(y);
+        assert (heights.get(y) > 0) : "Non-positive height: "+heights.get(y);
         hsum += heights.get(y);
       }
       assert (Math.abs(hsum - theight) < 1E-10);
@@ -459,6 +467,15 @@ public class RectangleArranger<T> {
     return Collections.unmodifiableSet(map.entrySet());
   }
 
+  /**
+   * The item keys contained in the map.
+   * 
+   * @return key set
+   */
+  public Set<T> keySet() {
+    return Collections.unmodifiableSet(map.keySet());
+  }
+
   /**
    * Test method.
    * 
