@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.svg;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -117,9 +117,19 @@ public class SVGPath {
     this.moveTo(x, y);
   }
 
+  /**
+   * Constructor with initial point.
+   * 
+   * @param xy initial coordinates
+   */
+  public SVGPath(double[] xy) {
+    this();
+    this.moveTo(xy);
+  }
+
   /**
    * Constructor from a vector collection (e.g. a polygon)
-   *
+   * 
    * @param vectors vectors
    */
   public SVGPath(Iterable<Vector> vectors) {
@@ -150,6 +160,25 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Draw a line given a series of coordinates.
+   * 
+   * Helper function that will use "move" for the first point, "lineto" for the
+   * remaining.
+   * 
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath drawTo(double[] xy) {
+    if(!isStarted()) {
+      moveTo(xy);
+    }
+    else {
+      lineTo(xy);
+    }
+    return this;
+  }
+
   /**
    * Draw a line given a series of coordinates.
    * 
@@ -190,6 +219,17 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Draw a line to the given coordinates.
+   * 
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath lineTo(double[] xy) {
+    append(SVGConstants.PATH_LINE_TO, xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Draw a line to the given coordinates.
    * 
@@ -213,6 +253,17 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Draw a line to the given relative coordinates.
+   * 
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath relativeLineTo(double[] xy) {
+    append(PATH_LINE_TO_RELATIVE, xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Draw a line to the given relative coordinates.
    * 
@@ -280,6 +331,17 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Move to the given coordinates.
+   * 
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath moveTo(double[] xy) {
+    append(SVGConstants.PATH_MOVE, xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Move to the given coordinates.
    * 
@@ -303,6 +365,17 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Move to the given relative coordinates.
+   * 
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath relativeMoveTo(double[] xy) {
+    append(PATH_MOVE_RELATIVE, xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Move to the given relative coordinates.
    * 
@@ -330,6 +403,19 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Cubic Bezier line to the given coordinates.
+   * 
+   * @param c1xy first control point
+   * @param c2xy second control point
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath cubicTo(double[] c1xy, double[] c2xy, double[] xy) {
+    append(SVGConstants.PATH_CUBIC_TO, c1xy[0], c1xy[1], c2xy[0], c2xy[1], xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Cubic Bezier line to the given coordinates.
    * 
@@ -359,6 +445,19 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Cubic Bezier line to the given relative coordinates.
+   * 
+   * @param c1xy first control point
+   * @param c2xy second control point
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath relativeCubicTo(double[] c1xy, double[] c2xy, double[] xy) {
+    append(PATH_CUBIC_TO_RELATIVE, c1xy[0], c1xy[1], c2xy[0], c2xy[1], xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Cubic Bezier line to the given relative coordinates.
    * 
@@ -386,6 +485,18 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Smooth Cubic Bezier line to the given coordinates.
+   * 
+   * @param c2xy second control point
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath smoothCubicTo(double[] c2xy, double[] xy) {
+    append(PATH_SMOOTH_CUBIC_TO, c2xy[0], c2xy[1], xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Smooth Cubic Bezier line to the given coordinates.
    * 
@@ -412,6 +523,18 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Smooth Cubic Bezier line to the given relative coordinates.
+   * 
+   * @param c2xy second control point
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath relativeSmoothCubicTo(double[] c2xy, double[] xy) {
+    append(PATH_SMOOTH_CUBIC_TO_RELATIVE, c2xy[0], c2xy[1], xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Smooth Cubic Bezier line to the given relative coordinates.
    * 
@@ -438,6 +561,18 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Quadratic Bezier line to the given coordinates.
+   * 
+   * @param c1xy first control point
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath quadTo(double[] c1xy, double[] xy) {
+    append(SVGConstants.PATH_QUAD_TO, c1xy[0], c1xy[1], xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Quadratic Bezier line to the given coordinates.
    * 
@@ -464,6 +599,18 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Quadratic Bezier line to the given relative coordinates.
+   * 
+   * @param c1xy first control point
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath relativeQuadTo(double[] c1xy, double[] xy) {
+    append(PATH_QUAD_TO_RELATIVE, c1xy[0], c1xy[1], xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Quadratic Bezier line to the given relative coordinates.
    * 
@@ -488,6 +635,17 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Smooth quadratic Bezier line to the given coordinates.
+   * 
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath smoothQuadTo(double[] xy) {
+    append(SVGConstants.PATH_SMOOTH_QUAD_TO, xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Smooth quadratic Bezier line to the given coordinates.
    * 
@@ -511,6 +669,17 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Smooth quadratic Bezier line to the given relative coordinates.
+   * 
+   * @param xy new coordinates
+   * @return path object, for compact syntax.
+   */
+  public SVGPath relativeSmoothQuadTo(double[] xy) {
+    append(PATH_SMOOTH_QUAD_TO_RELATIVE, xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Smooth quadratic Bezier line to the given relative coordinates.
    * 
@@ -528,7 +697,7 @@ public class SVGPath {
    * @param rx x radius
    * @param ry y radius
    * @param ar x-axis-rotation
-   * @param la large arc flag, if angle >= 180¬∞
+   * @param la large arc flag, if angle >= 180 deg
    * @param sp sweep flag, if arc will be drawn in positive-angle direction
    * @param x new coordinates
    * @param y new coordinates
@@ -538,12 +707,27 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Elliptical arc curve to the given coordinates.
+   * 
+   * @param rx x radius
+   * @param ry y radius
+   * @param ar x-axis-rotation
+   * @param la large arc flag, if angle >= 180 deg
+   * @param sp sweep flag, if arc will be drawn in positive-angle direction
+   * @param xy new coordinates
+   */
+  public SVGPath ellipticalArc(double rx, double ry, double ar, double la, double sp, double[] xy) {
+    append(SVGConstants.PATH_ARC, rx, ry, ar, la, sp, xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Elliptical arc curve to the given coordinates.
    * 
    * @param rxy radius
    * @param ar x-axis-rotation
-   * @param la large arc flag, if angle >= 180¬∞
+   * @param la large arc flag, if angle >= 180 deg
    * @param sp sweep flag, if arc will be drawn in positive-angle direction
    * @param xy new coordinates
    */
@@ -558,7 +742,7 @@ public class SVGPath {
    * @param rx x radius
    * @param ry y radius
    * @param ar x-axis-rotation
-   * @param la large arc flag, if angle >= 180¬∞
+   * @param la large arc flag, if angle >= 180 deg
    * @param sp sweep flag, if arc will be drawn in positive-angle direction
    * @param x new coordinates
    * @param y new coordinates
@@ -568,12 +752,27 @@ public class SVGPath {
     return this;
   }
 
+  /**
+   * Elliptical arc curve to the given relative coordinates.
+   * 
+   * @param rx x radius
+   * @param ry y radius
+   * @param ar x-axis-rotation
+   * @param la large arc flag, if angle >= 180 deg
+   * @param sp sweep flag, if arc will be drawn in positive-angle direction
+   * @param xy new coordinates
+   */
+  public SVGPath relativeEllipticalArc(double rx, double ry, double ar, double la, double sp, double[] xy) {
+    append(PATH_ARC_RELATIVE, rx, ry, ar, la, sp, xy[0], xy[1]);
+    return this;
+  }
+
   /**
    * Elliptical arc curve to the given relative coordinates.
    * 
    * @param rxy radius
    * @param ar x-axis-rotation
-   * @param la large arc flag, if angle >= 180¬∞
+   * @param la large arc flag, if angle >= 180 deg
    * @param sp sweep flag, if arc will be drawn in positive-angle direction
    * @param xy new coordinates
    */
