@@ -4,7 +4,7 @@ package de.lmu.ifi.dbs.elki.visualization.svg;
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
 
- Copyright (C) 2011
+ Copyright (C) 2012
  Ludwig-Maximilians-Universität München
  Lehr- und Forschungseinheit für Datenbanksysteme
  ELKI Development Team
@@ -23,9 +23,10 @@ package de.lmu.ifi.dbs.elki.visualization.svg;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import gnu.trove.map.hash.TObjectIntHashMap;
+
 import java.awt.Color;
 import java.text.NumberFormat;
-import java.util.HashMap;
 import java.util.Locale;
 
 import javax.swing.text.html.StyleSheet;
@@ -77,9 +78,16 @@ public final class SVGUtil {
   /**
    * SVG color names conversion.
    */
-  final private static HashMap<String, Integer> SVG_COLOR_NAMES = new HashMap<String, Integer>();
+  final private static TObjectIntHashMap<String> SVG_COLOR_NAMES;
+  
+  /**
+   * Key not found value. Not a reasonable color, fully transparent!
+   */
+  final private static int NO_VALUE = 0x00123456;
 
   static {
+    // Build a reasonably sized hashmap. Use 0
+    SVG_COLOR_NAMES = new TObjectIntHashMap<String>(90, .8f, NO_VALUE);
     // List taken from SVG specification:
     // http://www.w3.org/TR/SVG/types.html#ColorKeywords
     SVG_COLOR_NAMES.put("aliceblue", 0xFFF0F8FF);
@@ -229,6 +237,8 @@ public final class SVGUtil {
     SVG_COLOR_NAMES.put("whitesmoke", 0xFFF5F5F5);
     SVG_COLOR_NAMES.put("yellow", 0xFFFFFF00);
     SVG_COLOR_NAMES.put("yellowgreen", 0xFF9ACD32);
+    // Nonstandard:
+    SVG_COLOR_NAMES.put("transparent", 0xFFFFFFFF);
   }
 
   /**
@@ -468,8 +478,8 @@ public final class SVGUtil {
    * @return Color value
    */
   public static Color stringToColor(String str) {
-    Integer icol = SVG_COLOR_NAMES.get(str.toLowerCase());
-    if(icol != null) {
+    int icol = SVG_COLOR_NAMES.get(str.toLowerCase());
+    if(icol != NO_VALUE) {
       return new Color(icol, false);
     }
     return colorLookupStylesheet.stringToColor(str);
@@ -478,6 +488,8 @@ public final class SVGUtil {
   /**
    * Convert a color name from an AWT color object to CSS syntax
    * 
+   * Note: currently only RGB (from ARGB order) are supported.
+   * 
    * @param col Color value
    * @return Color string
    */
@@ -485,6 +497,18 @@ public final class SVGUtil {
     return String.format("#%02x%02x%02x", col.getRed(), col.getGreen(), col.getBlue());
   }
 
+  /**
+   * Convert a color name from an AWT color object to CSS syntax
+   * 
+   * Note: currently only RGB (from ARGB order) are supported.
+   * 
+   * @param col Color value
+   * @return Color string
+   */
+  public static String colorToString(int col) {
+    return String.format("#%02x%02x%02x", (col >>> 16) & 0xFF, (col >>> 8) & 0xFF, col & 0xFF);
+  }
+
   /**
    * Make a transform string to add margins
    * 
@@ -572,4 +596,62 @@ public final class SVGUtil {
       tag.removeChild(last);
     }
   }
+
+  /**
+   * Remove an element from its parent, if defined.
+   * 
+   * @param elem Element to remove
+   */
+  public static void removeFromParent(Element elem) {
+    if(elem != null) {
+      if(elem.getParentNode() != null) {
+        elem.getParentNode().removeChild(elem);
+      }
+    }
+  }
+
+  /**
+   * Create a circle segment.
+   * 
+   * @param svgp Plot to draw to
+   * @param centerx Center X position
+   * @param centery Center Y position
+   * @param angleStart Starting angle
+   * @param angleDelta Angle delta
+   * @param innerRadius inner radius
+   * @param outerRadius outer radius
+   * @return SVG element representing this circle segment
+   */
+  public static Element svgCircleSegment(SVGPlot svgp, double centerx, double centery, double angleStart, double angleDelta, double innerRadius, double outerRadius) {
+    double sin1st = Math.sin(angleStart);
+    double cos1st = Math.cos(angleStart);
+  
+    double sin2nd = Math.sin(angleStart + angleDelta);
+    double cos2nd = Math.cos(angleStart + angleDelta);
+  
+    double inner1stx = centerx + (innerRadius * sin1st);
+    double inner1sty = centery - (innerRadius * cos1st);
+    double outer1stx = centerx + (outerRadius * sin1st);
+    double outer1sty = centery - (outerRadius * cos1st);
+  
+    double inner2ndx = centerx + (innerRadius * sin2nd);
+    double inner2ndy = centery - (innerRadius * cos2nd);
+    double outer2ndx = centerx + (outerRadius * sin2nd);
+    double outer2ndy = centery - (outerRadius * cos2nd);
+  
+    double largeArc = 0;
+    if(angleDelta >= Math.PI) {
+      largeArc = 1;
+    }
+  
+    SVGPath path = new SVGPath(inner1stx, inner1sty);
+    path.lineTo(outer1stx, outer1sty);
+    path.ellipticalArc(outerRadius, outerRadius, 0, largeArc, 1, outer2ndx, outer2ndy);
+    path.lineTo(inner2ndx, inner2ndy);
+    if (innerRadius > 0) {
+      path.ellipticalArc(innerRadius, innerRadius, 0, largeArc, 0, inner1stx, inner1sty);
+    }
+  
+    return path.makeElement(svgp);
+  }
 }
\ No newline at end of file
